
!DIR$ FREE
!CRAY - Add source form directive
module input_output_units
!
!        nout is the unit for output
!        nin is the unit for input
!        spfile is the specular reflection file
!
    integer, parameter :: nin=3,nout=6,spfile=11
end module input_output_units


!DIR$ FREE
!CRAY - Add source form directive
module kind_definitions
    integer, parameter :: LONGreal = selected_real_kind(13,90)  
end module kind_definitions


!DIR$ FREE
!CRAY - Add source form directive
module free_format_parser_variables
    integer icpnt
    logical eoff
    character card*80,field*80
    save icpnt,eoff,card,field
end module free_format_parser_variables


!DIR$ FREE
!CRAY - Add source form directive
module constants
!
    use kind_definitions
!
    integer, parameter:: LONGreal2 = LONGreal 
!
!        hbar  = Planck's constant over two pi (erg-sec)
!        small = a constant which will be used whenever a small
!                number is required
!        big   = a constant which will be used whenever a large
!                number is required
!        conve = conversion from mev to ergs
!        convl = conversion from angstroms to cm
!        convm = conversion from amu to gm
!        dtr   = conversion from degrees to radians
!
!
    real (kind=LONGreal2), parameter :: hbar=1.05459e-27,small=1.e-10,pi=3.1415926,                &
&                                       twopi=6.2831853,conve=1.602e-15,convl=1.e-8,               &
&                                       convm=1.660565e-24,dtr=1.7453292e-2,big=1.e30
!
end module constants

!DIR$ FREE
!CRAY - Add source form directive

program denali
!
!        Copyright (C) 1991 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Calculate t matrix elements for the quantum mechanical
!                     scattering of low energy atoms from crystal surfaces
!        CODED:       Serial version coded 1 November 1990.
!                     CM version coded 31 August 1991
!                     F90 version coded 14 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Formerly:  Department of Physics and Astronomy,
!                                University of New Mexico
!                     Currently: Quetzal Computational Associates
!                                3200 Carlisle N.E.
!                                Albuquerque, NM   87110-1664
!
!        For an explanation of this code and the underlying physics
!        and mathematics, see "A Quantum Mechanical Theory for the
!        Scattering of Low-Energy Atoms from Incommensurate Crystal
!        Surface Layers", by John K. Prentice, PhD dissertation,
!        University of New Mexico, available from UMI Dissertation
!        Information Service, order number 9302428. 
!
    use kind_definitions
    use constants
    use input_output_units
!
    implicit none
!
    real (kind = LONGreal), dimension(:,:), allocatable :: pin
    real (kind = LONGreal), dimension(:), allocatable :: zlayer,zcoeff
    real (kind = LONGreal), dimension(:), allocatable :: pterm
    real (kind = LONGreal), dimension(:), allocatable :: tinnow,phinow
    logical brief,bounce
    complex (kind = LONGreal), dimension(:), allocatable :: wght,yvectr
    complex (kind = LONGreal), dimension(:,:), allocatable :: matrix
    real (kind = LONGreal), dimension(:), allocatable :: points
    integer, dimension(:), allocatable :: mindex,nindex
    integer, dimension(:,:), allocatable :: mnindx
    integer :: order,nvirtl,nspec,nnspec,ptype,qtype,number_of_crystal_layers,nphys,nvirt,nvknt,   &
&              nnsknt,nsknt,number_of_potential_terms,number_of_momentum_values,n,                 &
&              maximum_order,ncalc,ilast,maximum_physical
    real (kind=LONGreal) :: tstart,tstop,dt,pmass,x_lattice_constant,y_lattice_constant,ttotal,zmu,&
&                           smom,vfactr,lambda
    character output_file_name*80,sp_file_name*80
!
    call second (tstart)
!
!**************** scan input and allocate some memory ******************
!
    call initial_input (number_of_crystal_layers,number_of_potential_terms,number_of_momentum_values)
!
    allocate (pin(3,number_of_momentum_values))
    allocate (zlayer(number_of_crystal_layers))
    allocate (zcoeff(number_of_crystal_layers))
    allocate (pterm(number_of_potential_terms))
    allocate (tinnow(number_of_momentum_values))
    allocate (phinow(number_of_momentum_values))
!
!****** rescan input, initialize the code, and allocate some more memory *******
!
    call final_input (pmass,brief,number_of_crystal_layers,zlayer,zcoeff,vfactr,x_lattice_constant,&
&                     y_lattice_constant,lambda,smom,zmu,pin,ptype,pterm,number_of_potential_terms,&
&                     nvknt,nsknt,nnsknt,qtype,nvirtl,nspec,nnspec,bounce,number_of_momentum_values&
&                     ,tinnow,phinow,output_file_name)
!  
!        calculate the matrix order and allocate memory
!
    maximum_order=0
    maximum_physical = 0
    do n=1,number_of_momentum_values
        call numpk (vfactr,x_lattice_constant,y_lattice_constant,pin(1,n),nphys,nvirt)
        if (qtype.eq.1) then
            order=2*(nspec+1+(nphys-1)*(nnspec+1)+nvirt*nvirtl)
        else
            order=2*(nsknt+(nphys-1)*nnsknt+nvirt*(nvknt-1))+nphys+nvirt
        endif
        maximum_order=max(maximum_order,order)
        maximum_physical = max(maximum_physical,nphys)
!
        write (nout,'(/'' calculation number '',i5,'' theta='',1pe12.5,'' phi='',e12.5)')          &
&                     n,tinnow(n),phinow(n)
        write (nout,'(/'' number of physical reciprocal lattice vectors = '',i5/                   &
&                      '' number of virtual reciprocal lattice vectors = '',i5/                    &
&                      '' order of matrix = '',i10/)')nphys,nvirt,order
    enddo
!
    allocate (wght(1:maximum_order))
    allocate (yvectr(1:maximum_order))
    allocate (matrix(1:maximum_order,1:maximum_order))
    allocate (points(1:maximum_order))
    allocate (mindex(1:maximum_order))
    allocate (nindex(1:maximum_order))
    allocate (mnindx(1:2,1:maximum_physical))
!
    call second (tstop)
    dt=tstop-tstart
    ttotal=dt
    write(nout,'(/'' time to read input = '',1pe12.5,'' seconds''/)') dt
    tstart=tstop
!
!        open specular output file
!
    do n=80,1,-1
        ilast=n
         if (output_file_name(n:n).ne.' ') exit
    enddo
    sp_file_name=output_file_name(1:ilast)//'.sp'
    sp_file_name='scatteri.sp'
    open (unit=spfile,status='unknown',file=sp_file_name,form='formatted')
!
!        loop over calculations
!
    do ncalc=1,number_of_momentum_values
        write(nout,'(///'' ********* Calculation '',i5, '' tin='',1pe12.5,'' phi='',e12.5)')       &
&                      ncalc,tinnow(ncalc),phinow(ncalc)
!
        call numpk (vfactr,x_lattice_constant,y_lattice_constant,pin(1,ncalc),nphys,nvirt)
!
        if (qtype.eq.1) then
            order=2*(nspec+1+(nphys-1)*(nnspec+1)+nvirt*nvirtl)
        else
            order=2*(nsknt+(nphys-1)*nnsknt+nvirt*(nvknt-1))+nphys+nvirt
        endif
        write(nout,'('' matrix order='',i7)') order
!
!        ********* generate momentum partition locations (knots) *******
!
        if (qtype.eq.1) then
            call mkchev (vfactr,x_lattice_constant,y_lattice_constant,pin(1,ncalc),brief,pmass,    &
&                        smom,order,nspec,nnspec,nvirtl,points(1:order),wght(1:order),             &
&                        mindex(1:order),nindex(1:order),nphys,mnindx)
        elseif (qtype.eq.2) then
            call mkcube (vfactr,x_lattice_constant,y_lattice_constant,pin(1,ncalc),brief,pmass,    &
&                        smom,order,nsknt,nnsknt,nvknt,points(1:order),wght(1:order),              &
&                        mindex(1:order),nindex(1:order),nphys,mnindx)
        endif
!
        call second (tstop)
        dt=tstop-tstart
        ttotal=ttotal+dt
        write(nout,'(/'' time to generate partition points = '',1pe12.5,'' seconds''/)') dt
        tstart=tstop
!
!        ********************** build matrix to solve ******************
!
        call build (maximum_order,order,points,wght,mindex,nindex,lambda,x_lattice_constant,       &
&                   y_lattice_constant,number_of_crystal_layers,zlayer,zcoeff,matrix,ptype,pterm,  &
&                   number_of_potential_terms,bounce)
        call second (tstop)
        dt=tstop-tstart
        ttotal=ttotal+dt
        write(nout,'(/'' time to build matrix = '',1pe12.5,'' seconds''/)') dt
        tstart=tstop
!
!        ************************ make plots ***************************
!
        call doplot (maximum_order,order,matrix)
    enddo
!
!        ************* print total time for calculation ****************
!
    write(nout,'(//'' total time for calculation = '',1pe12.5,'' seconds''/)') ttotal
    close (unit=4)
    close (unit=spfile)
!
end program denali


!DIR$ FREE
!CRAY - Add source form directive
subroutine bterm (mindex,nindex,q,p,nterms,lambda,xlatt,ylatt,nlayer,zlayer,zcoeff,term,           &
&                 poten,pterm,npterm,bounce)
!
!
!        Copyright (C) 1991 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     calculate the B  (q,p) term 
!                                    mn
!        CODED:       CM version coded August 1991
!                     F90 version coded 14 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Formerly:  Department of Physics and Astronomy,
!                                University of New Mexico
!                     Currently: Quetzal Computational Associates
!                                3200 Carlisle N.E.
!                                Albuquerque, NM   87110-1664
!
!
use kind_definitions
use constants
use input_output_units
!
implicit none
!
integer :: npterm,nterms,nlayer
real (kind=LONGreal), parameter :: zero=0.0,one=1.0,onept5=1.5,two=2.0,four=4.0,eight=8.0,         &
&                                  pt25=0.25,pt5=0.5,three=3.0,three60=360.0,five=5.0,             &
&                                  twenty_four=24.0,six=6.0,seven=7.0,forty_eight=48.0,            &
&                                  thirteen=13.0,seventy_two=72.0,twenty_five=25.0,                &
&                                  two_hundred_eighty_eight=288.0,one_hundred_twenty=120.0,        &
&                                  seven_hundred_twenty=720.0
logical bounce
real (kind=LONGreal), dimension(nterms) :: work1,work2,q,p
real (kind=LONGreal), dimension(nlayer) :: zlayer,zcoeff
real (kind=LONGreal), dimension(npterm) :: pterm
real (kind=LONGreal) :: xlatt,ylatt,sigsq,expon,const,const2,rlamda,coeff,xreal,yreal,const1,      &
&                       ali,af,bli,bf,cli,cf,lambda
complex (kind=LONGreal), dimension(nterms) :: term,ccoeff,ecoeff,literm,fterm,sum,zf,zli,          &
&                                             log_zfm1,log_zlim1
complex (kind=LONGreal) :: arg,zconst1,zconst2,zconst3,zconst4,zconst5,zconst6,zconst7,zconst8,    &
&                          zconst9,zconst10,zconst11
integer, dimension(nterms) :: mindex,nindex
integer :: poten,nl
!
coeff=(xlatt*ylatt)/twopi**onept5
rlamda=one/(four*lambda)
!
!        calculate delta vector magnitude and store in work array
!
work1=sqrt((xlatt*real(mindex,LONGreal))**2+(ylatt*real(nindex,LONGreal))**2+(q-p)**2)
work2=q-p
!
!        calculate Fourier transform of potential
!
if (poten.eq.1) then
!
!        Yukawa potential Fourier transform calculation
!
    const=sqrt(two/pi)*pterm(1)
    sigsq=one/pterm(2)**2
    term=cmplx(const/(sigsq+work1**2),zero,LONGreal)
elseif (poten.eq.2) then
!
!        Morse potential Fourier transform calculation
!
    expon=exp(pterm(2)*pterm(3))
    const=pterm(1)*sqrt(two/pi)*expon
    const2=two*sqrt(two/pi)*pterm(1)*expon
    where ( work1.gt.small )
        term=cmplx(const/work1*(expon*sin(two*atan(work1/(two*pterm(2))))/four*pterm(2)**2+       &
&                work1**2-two*sin(two*atan(work1/pterm(2)))/pterm(2)**2+work1**2),zero,LONGreal)
    elsewhere
        term=cmplx(const2*(expon/(eight*pterm(2)**3)-two/pterm(2)**3),zero,LONGreal)
    endwhere
elseif (poten.eq.3) then
!
!        repulsive square barrier potential
!
    const=sqrt(two/pi)*pterm(1)
    where ( abs(work2).gt.small )
        term=cmplx(const/work2*sin(pt5*pterm(2)*work2),zero,LONGreal)
    elsewhere
        term=cmplx(pt5*const*pterm(2),zero,LONGreal)
    endwhere
    coeff=one
elseif (poten.eq.4) then
!
!        Tang/Toennies Hi-LiF unit cell potential
!
    ali=pterm(1)
    af=pterm(4)
    bli=pterm(2)
    bf=pterm(5)
    cli=pterm(3)
    cf=pterm(6)
    xreal=twopi/xlatt
    yreal=twopi/ylatt
    if (abs(xreal-yreal).gt.small) then
        write(nout,'(/'' LiF lattice constants are not the same, abort in bterm''/)')
        call abort
    endif
    const1=sqrt(two/pi)
    const2=(two)**onept5/sqrt(pi)
    arg=cmplx(zero,-xreal/sqrt(two),LONGreal)
    zconst1=cmplx(-one/twenty_four,zero,LONGreal)
    zconst2=cmplx(one/six,zero,LONGreal)
    zconst3=cmplx(-pt25,zero,LONGreal)
    zconst4=zconst2
    zconst5=zconst1
    zconst6=zconst1
    zconst7=cmplx(seven/forty_eight,zero,LONGreal)
    zconst8=cmplx(-thirteen/seventy_two,zero,LONGreal)
    zconst9=cmplx(twenty_five/two_hundred_eighty_eight,zero,LONGreal)
    zconst10=cmplx(-one/one_hundred_twenty,zero,LONGreal)
    zconst11=cmplx(-one/seven_hundred_twenty,zero,LONGreal)
!
!        zero results
!
    term=cmplx(zero,zero,LONGreal)
!
!        calculate unit surface cell coefficients
!
    where ( mod(mindex+nindex,2).eq.0 )
        ccoeff=cmplx(one,zero,LONGreal)
    elsewhere
        ccoeff=-cmplx(one,zero,LONGreal)
    endwhere
!
!        calculate l=2 coefficients
!
    ecoeff=exp(arg*cmplx(work2,zero,LONGreal))
!
!        calculate z (needded for attractive term)
!
    zf=cmplx(one,zero,LONGreal)/cmplx(one,-work1/bf,LONGreal)
    zli=cmplx(one,zero,LONGreal)/cmplx(one,-work1/bli,LONGreal)
!
!        do repulsive term first
!
!
!        non-zero momentum case
!
    where (work1.gt.small)
        literm=cmplx(const1*ali/(work1*(work1**2+bli**2))*sin(two*atan(work1/bli)),zero,LONGreal)
        fterm=cmplx(const1*af/(work1*(work1**2+bf**2))*sin(two*atan(work1/bf)),zero,LONGreal)
!
!        zero momentum case
!
    elsewhere
        literm=cmplx(const2*(ali/bli**3),zero,LONGreal)
        fterm=cmplx(const2*(af/bf**3),zero,LONGreal)
    endwhere
!
!
!        odd numbered layer
!
    term=term+literm+ccoeff*fterm
!
!        even numbered layer
!
    term=term+(literm*ccoeff+fterm)*ecoeff
!
!        calculate ln(1-z) for use in attractive part calculation
!
    where (work1.gt.small)
        log_zfm1=log(cmplx(one,zero,LONGreal)-zf)
        log_zlim1=log(cmplx(one,zero,LONGreal)-zli)
    endwhere
!
!        evaluate attractive part second
!
!
!        non-zero momentum case
!
    where ( work1.gt.small )
        literm=cmplx(const1*cli*bli**four/work1*aimag((log_zlim1*(zconst1+zli*zconst2+            &
&                  zli**two*zconst3+zli**three*zconst4+zli**four*zconst5)+zli*(zconst6+            &
&                  zli*zconst7+zli**two*zconst8+zli**three*zconst9+zli**four*zconst10+             &
&                  zli**five*zconst11))/zli**four),zero,LONGreal)
        fterm=cmplx(const1*cf*bf**four/work1*aimag((log_zfm1*(zconst1+zf*zconst2+                 &
&                  zf**two*zconst3+zf**three*zconst4+zf**four*zconst5)+zf*(zconst6+                &
&                  zf*zconst7+zf**two*zconst8+zf**three*zconst9+zf**four*zconst10+                 &
&                  zf**five*zconst11))/zf**four),zero,LONGreal)
!
!        zero momentum case
!
    elsewhere
        literm=cmplx((const1*cli*bli**three)/three60,zero,LONGreal)
        fterm=cmplx((const1*cf*bf**three)/three60,zero,LONGreal)
    endwhere
!
!        odd numbered layer
!
    term=term-(literm+ccoeff*fterm)
!
!        even numbered layer
!
    term=term-(literm*ccoeff+fterm)*ecoeff
!
else
    write(nout,'(/'' unsupported potential in bterm, abort''/'' requested potential = '',i3/)') poten
    call abort
endif
!
!        put in phase factor associated with the position of the layers
!
if(nlayer.gt.1) then
    sum=cmplx(zero,zero,LONGreal)
!
    do nl=1,nlayer
        sum=sum+zcoeff(nl)*exp(cmplx(zero,work2*zlayer(nl),LONGreal))
    enddo
!
    term=term*sum
else
    term=term*exp(cmplx(zero,work2*zlayer(1),LONGreal))
endif
!
!        finish calculation
!
if(.not.bounce) then
    term=coeff*term
else
    term=coeff*term*exp(-rlamda*work1**2)
endif
!
!
end subroutine bterm



!DIR$ FREE
!CRAY - Add source form directive
subroutine initial_input (number_of_crystal_layers,number_of_potential_terms,                      &
&                         number_of_momentum_values)
!
!        Copyright (C) 1993 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Scan input for information necessary to perform
!                     memory allocations
!        CODED:       F90 version coded 14 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Quetzal Computational Associates
!                     3200 Carlisle N.E.
!                     Albuquerque, NM   87110-1664
!
use kind_definitions
use input_output_units
use free_format_parser_variables
!
implicit none
!
real ( kind=LONGreal ), parameter :: pt5=0.5
!
integer :: number_of_crystal_layers,number_of_potential_terms,number_of_momentum_values,numtin,    &
&          numphi,ier
logical :: azim,polar
real (kind = LONGreal) :: zval
external value,next,error
!
!CRAY - Change name for test driver
open (unit=nin,file='scatteri.in',status='old')
rewind nin
!
icpnt = 9999
number_of_crystal_layers = 0
number_of_potential_terms = 0
polar = .false.
azim = .false.
!
!        parse the input
!
10 call next
   if(eoff) then
       close(unit=nin)
       go to 60
   endif
20 if (field.eq.'potential_terms') then
30     call value(zval,ier)
       if(ier.eq.-1) then
           if (number_of_potential_terms.eq.0) then
               write(nout,'(/'' expecting potential terms, but none found, abort '')')
               call abort
           endif
           close (unit=nin)
           go to 60
       else if (ier.eq.0) then
           if (number_of_potential_terms.eq.0) then
               write(nout,'(/'' expecting potential terms, but none found, abort '')')
               call abort
           else
               go to 20
           endif
       else
           number_of_potential_terms=number_of_potential_terms+1
           go to 30
       endif
   endif
!
40 if (field.eq.'z_layer_locations') then
50     call value(zval,ier)
       if(ier.eq.-1) then
           if (number_of_crystal_layers.eq.0) then
               write(nout,'(/'' expecting layer locations, but none found, abort '')')
               call abort
           endif
           close (unit=nin)
           go to 60
       else if (ier.eq.0) then
           if (number_of_crystal_layers.eq.0) then
               write(nout,'(/'' expecting layer locations, but none found, abort '')')
               call abort
           else
               go to 40
           endif
       else
           number_of_crystal_layers=number_of_crystal_layers+1
           go to 50
       endif
!
    elseif (field.eq.'number_of_incoming_polar_angles') then
        polar=.true.
        call value (zval,ier)
        numtin = int(zval + pt5)
        if (ier.le.0) call error (card,field,ier)
!
    elseif (field.eq.'initial_incoming_azimuthal_angle') then
        azim=.true.
!
    elseif (field.eq.'final_incoming_azimuthal_angle') then
        azim=.true.
!
    elseif (field.eq.'number_of_incoming_azimuthal_angles') then
        azim=.true.
        call value (zval,ier)
        numphi=int(zval+pt5)
        if (ier.le.0) call error (card,field,ier)
!
    endif
!
    go to 10
!
60  if (.not.polar) numtin=1
    if (.not.azim) numphi=1
    number_of_momentum_values= numtin*numphi
!
end subroutine initial_input


!DIR$ FREE
!CRAY - Add source form directive
subroutine final_input (pmass,brief,maxlay,zlayer,zcoeff,vfactr,xlatt,ylatt,lambda,smom,zmu,pin,   &
&                       ptype,pterm,mxterm,nvknt,nsknt,nnsknt,qtype,nvirtl,nspec,nnspec,bounce,    &
&                       maxpin,tinnow,phinow,output_file_name)
!
!        Copyright (C) 1993 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Scan input for calculational input
!        CODED:       F90 version coded 14 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Quetzal Computational Associates
!                     3200 Carlisle N.E.
!                     Albuquerque, NM   87110-1664
!
use kind_definitions
use input_output_units
use free_format_parser_variables
use constants
!
implicit none
!
real (kind = LONGreal), parameter :: zero=0.0,one=1.0,ten8=1.e8,two=2.0,onee3=1.e3,pt5=0.5
integer, parameter :: ilarge=99999
integer :: ptype,qtype,maxlay,nlayer,npterm,mxterm,nvknt,nsknt,nnsknt,nvirtl,nspec,nnspec,ier,     &
&          nstren,n,maxpin,numpin,numtin,numphi,it,ip
real (kind = LONGreal), dimension(maxpin) :: tinnow,phinow
real (kind = LONGreal), dimension(maxlay) :: zlayer,zcoeff
real (kind = LONGreal), dimension(mxterm) :: pterm
real (kind = LONGreal), dimension(3,maxpin) :: pin
real (kind = LONGreal) :: lambda,pmass,vfactr,xlatt,ylatt,smom,zmu,ein,smass,slen,pmom,wvin,a,b,   &
&                         tmass,omega,sener,zval,tin,phiin,dmy,prob,debrog,tinin,tinout,      &
&                         phiout,tinnum,phinum,delphi,deltin,phi
logical :: brief,first,bounce,polar,azim
character output_file_name*80
external value,next,error
!
!CRAY change name of input file for test driver
open (unit=nin,file='scatteri.in',status='old')
rewind nin
	delphi=0.0	! JRA
	deltin=0.0	! JRA
icpnt=9999
nlayer=0
nstren=0
brief=.false.
prob=zero
vfactr=one
first=.true.
nvirtl=0
nspec=0
nnspec=0
nvknt=0
nsknt=0
nnsknt=0
ptype=0
npterm=0
qtype=0
nvirtl=0
nspec=0
nnspec=0
numpin=0
polar=.false.
azim=.false.
tinin=ten8
tinout=ten8
numtin=-ilarge
phiin=ten8
phiout=ten8
numphi=-ilarge
bounce=.false.
!
!        parse the input
!
10 call next
if(eoff) then
    close(unit=nin)
    go to 50
endif
!
!        see if an output file is given
!
if(first) then
    if(field.eq.'output_file_name') then
        call next
!        open (unit=nout,file='scatteri.out',status='unknown')
        output_file_name='scatteri.out'
        first=.false.
!        open (unit=nout,file=field,status='unknown')
!        first=.false.
!        output_file_name=field
        go to 10
    else
!        open (unit=nout,file='scatteri.out',status='unknown')
        output_file_name='scatteri.out'
        first=.false.
    endif
endif
!
16 if (field.eq.'potential_terms') then
17     call value(zval,ier)
    if(ier.eq.-1) then
        if (npterm.eq.0) then
            write(nout,'(/'' expecting potential terms, but none found, abort '')')
            go to 40
        endif
        close (unit=nin)
        go to 50
    else if (ier.eq.0) then
        if (npterm.eq.0) then
            write(nout,'(/'' expecting potential terms, but none found, abort '')')
            go to 40
        else
            go to 16
        endif
    else
        npterm=npterm+1
        if(npterm.gt.mxterm) then
            write(nout,'(/'' npterm /= mxterm in final_input, abort.''/)')
            go to 40
        endif
        pterm(npterm)=zval
        go to 17
    endif
endif
!
20 if (field.eq.'z_layer_locations') then
21     call value(zval,ier)
    if(ier.eq.-1) then
        if (nlayer.eq.0) then
            write(nout,'(/'' expecting layer locations, but none found, abort '')')
            go to 40
        endif
        close (unit=nin)
        go to 50
    else if (ier.eq.0) then
        if (nlayer.eq.0) then
            write(nout,'(/'' expecting layer locations, but none found, abort '')')
            go to 40
        else
            go to 20
        endif
    else
        nlayer=nlayer+1
        if(nlayer.gt.maxlay) then
            write(nout,'(/'' nlayer /= maxlay in final_input, abort.''/)')
            go to 40
        endif
        zlayer(nlayer)=zval
        go to 21
    endif
!
elseif (field.eq.'z_layer_strength') then
22     call value(zval,ier)
    if(ier.eq.-1) then
        if (nstren.eq.0) then
            write(nout,'(/'' expecting layer strength coefficients, but none found, abort '')')
            go to 40
        endif
        close (unit=nin)
        go to 50
    else if (ier.eq.0) then
        if (nstren.eq.0) then
            write(nout,'(/'' expecting layer strength coefficients, but none found, abort '')')
            go to 40
        else
            go to 20
        endif
    else
        nstren=nstren+1
        if(nstren.gt.maxlay) then
            write(nout,'(/'' nstren /= maxlay in final_input, abort.''/)')
            go to 40
        endif
        if(zval.le.zero.or.zval.gt.one) then
            write(nout,'(/'' illegal value for the strength coefficient number '',i5/              &
&                         '' the illegal value was '',1pe12.5)') nstren,zval
            write(nout,'('' this coefficient must be greater than zero and less than or equal to one, abort'')')
            go to 40
        endif
        zcoeff(nstren)=zval
        go to 22
    endif
!
elseif (field.eq.'target_mass') then
    call value(tmass,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'projectile_mass') then
    call value(pmass,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'oscillator_frequency') then
    call value (omega,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'problem_number') then
    call value (prob,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'number_of_virtual_quadrature_points') then
    call value (zval,ier)
    nvirtl=int(zval)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'number_of_specular_quadrature_points') then
    call value (zval,ier)
    nspec=int(zval)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'number_of_non-specular_quadrature_points') then
    call value (zval,ier)
    nnspec=int(zval)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'number_of_virtual_quadrature_knots') then
    call value (zval,ier)
    nvknt=int(zval)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'number_of_specular_quadrature_knots') then
    call value (zval,ier)
    nsknt=int(zval)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'number_of_non-specular_quadrature_knots') then
    call value (zval,ier)
    nnsknt=int(zval)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'potential_type') then
    call value(dmy,ier)
    if (ier.lt.0) call error (card,field,ier)
    if (field.eq.'yukawa') then
        ptype=1
    elseif (field.eq.'morse') then
        ptype=2
    elseif (field.eq.'repulsive_square_barrier') then
        ptype=3
    elseif (field.eq.'Tang_Toennies_He-LiF_potential') then
        ptype=4
    else
        write(nout,'('' expected potential type on input and instead encountered the word: ''      &
&                    /1x,a80/'' abort''/)') field
        write(nout,'(/'' was reading the input line: ''/1x,a80)') card
        go to 40
    endif
!
elseif (field.eq.'interpolation_method') then
    call value(dmy,ier)
    if (ier.lt.0) call error (card,field,ier)
    if (field.eq.'chebyshev_polynomials') then
        qtype=1
    elseif (field.eq.'hermite_cubic_polynomials') then
        qtype=2
    else
        write(nout,'('' expected interpolation type on input and instead encountered the word: ''  &
&                    /1x,a80/'' abort''/)') field
        write(nout,'(/'' was reading the input line: ''/1x,a80)') card
        go to 40
    endif
!
elseif (field.eq.'incoming_wave_vector') then
    call value (wvin,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'incoming_polar_angle') then
    call value (tin,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'incoming_azimuthal_angle') then
    call value (phiin,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'initial_incoming_polar_angle') then
    polar=.true.
    call value (tinin,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'final_incoming_polar_angle') then
    polar=.true.
    call value (tinout,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'number_of_incoming_polar_angles') then
    polar=.true.
    call value (tinnum,ier)
    numtin=int(tinnum+pt5)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'initial_incoming_azimuthal_angle') then
    azim=.true.
    call value (phiin,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'final_incoming_azimuthal_angle') then
    azim=.true.
    call value (phiout,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'number_of_incoming_azimuthal_angles') then
    azim=.true.
    call value (phinum,ier)
    numphi=int(phinum+pt5)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'x_axis_lattice_constant') then
    call value (a,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'y_axis_lattice_constant') then
    call value (b,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'virtual_lattice_factor') then
    call value(vfactr,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'mass_scale_value') then
    call value(smass,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'length_scale_value') then
    call value(slen,ier)
    if (ier.le.0) call error (card,field,ier)
!
elseif (field.eq.'crystal_is_harmonic') then
    bounce=.true.
!
elseif (field.eq.'generate_minimal_output') then
    brief=.true.
!
else
!
!        unrecognized variable, abort
!
    write(nout,'('' the input variable : ''/1x,a80/'' is unrecognized, abort'')') field
    write(nout,'(/'' was reading the input line: ''/1x,a80)') card
    go to 40
endif
!
go to 10
!
!        close input file and abort
!
40 close (unit=nin)
call abort
return
!
!        make minimal validity checks
!
!
50 if(qtype.eq.1.and.nvirtl.le.0) then
    write(nout,'(/'' illegal value for number of virtual quadrature points, abort'')')
    go to 40
elseif(qtype.eq.1.and.nspec.le.0) then
    write(nout,'(/'' illegal value for number of specular quadrature points, abort'')')
    go to 40
elseif(qtype.eq.1.and.nnspec.le.0) then
    write(nout,'(/'' illegal value for number of non-specular quadrature points, abort'')')
    go to 40
endif
if(qtype.eq.2.and.nvknt.le.0) then
    write(nout,'(/'' illegal value for number of virtual quadrature knots, abort'')')
    go to 40
endif
if(qtype.eq.2.and.nsknt.le.0) then
    write(nout,'(/'' illegal value for number of specular quadrature knots, abort'')')
    go to 40
endif
if(qtype.eq.2.and.nnsknt.le.0) then
    write(nout,'(/'' illegal value for number of non-specular quadrature knots, abort'')')
    go to 40
endif
!
if(nlayer.eq.0) then
    zlayer(1)=zero
    zcoeff(1)=one
    nlayer=1
elseif (nstren.eq.0) then
    do 32 n=1,nlayer
        zcoeff(n)=one
32     continue
elseif (nstren.ne.nlayer) then
    write(nout,'(/'' there were '',i2,'' layers, but '',i2,'' strength coefficients specified.''/  &
&                 '' there must be the same number, abort ''/)') nlayer,nstren
    call abort
endif
!
if(vfactr.lt.one) then
    write(nout,'(/'' virtual lattice factor is less than one.''/'' this number must be one or larger, abort''/)')
    go to 40
endif
!
if(ptype.le.0) then
    write(nout,'(/'' no potential type specified, abort''/)')
    go to 40
endif
!
if(qtype.le.0) then
    write(nout,'(/'' no interpolation type specified, abort''/)')
    go to 40
endif
!
!        print out input values
!
write(nout,'('' Solution by Nystrom method with product integration''//'' problem number = '',     &
&            f10.5//)') prob
write(nout,'(/'' Fortran 90 Version''/)')
!
write(nout,'('' LU decomposition matrix solution method''/)')
!
if (bounce) then
    write(nout,'('' target mass (amu) = '',1pe12.5/'' projectile mass (amu) = '',1pe12.5/          &
&                '' oscillator frequency (radians/sec) = '',1pe12.5)') tmass,pmass,omega
else
    write(nout,'('' target mass (amu) = '',1pe12.5/'' projectile mass (amu) = '',1pe12.5/          &
&                '' lattice atoms are stationary '')') tmass,pmass
endif
!
!        convert wave vector to momentum (cgs) and energy (mev)
!
pmom=wvin*ten8*hbar
ein=pmom**2/(two*pmass*convm*conve)
debrog=twopi/wvin
!
write(nout,'('' incoming wave vector (1/angstroms) = '',1pe12.5/                                   &
&            '' incoming de Broglie wavelength (angstroms) = '',1pe12.5/                           &
&            '' incoming momentum (ergs/gm) = '',1pe12.5/'' incoming energy (mev) = '',1pe12.5)')  &
&            wvin,debrog,pmom,ein
if (.not.polar) then
    write(nout,'('' incoming theta = '',1pe12.5)') tin
else
    write(nout,'('' initial incoming theta = '',1pe12.5)') tinin
    write(nout,'('' final incoming theta = '',1pe12.5)') tinout
    write(nout,'('' number of theta angles = '',i5)') numtin
endif
if (.not.azim) then
    write(nout,'('' incoming phi = '',1pe12.5)') phiin
else
    write(nout,'('' initial incoming phi = '',1pe12.5)') phiin
    write(nout,'('' final incoming phi = '',1pe12.5)') phiout
    write(nout,'('' number of phi angles = '',i5)') numphi
endif
if (ptype.eq.1) then
    write(nout,'(/'' Yukawa repulsive potential''/                                                 &
&                 ''           potential coefficient (mev-ang) = '',1pe12.5/                       &
&                 ''           potential exponent (angstroms)  = '',1pe12.5/)') pterm(1),pterm(2)
elseif (ptype.eq.2) then
    write(nout,'(/'' Morse potential''/''           well depth (mev)                    = '',      &
&                 1pe12.5,''           reciprocal range parameter (1/ang)  = '',1pe12.5/           &
&                 ''           position of potential minimum (ang) = '',1pe12.5/))') pterm(1),     &
&                 pterm(2),pterm(3)
elseif (ptype.eq.3) then
    write(nout,'(/'' Square repulsive barrier potential''/                                         &
&                 ''           barrier height (mev)                = '',1pe12.5/                   &
&                 ''           barrier width (angstroms)           = '',1pe12.5/)') pterm(1),pterm(2)
elseif (ptype.eq.4) then
    if (npterm.ne.6) then
        write(nout,'(/'' wrong number of terms for Tang/Toennies He-LiF potential, abort''/)')
        call abort
    endif
!
    write(nout,'(/'' Tang/Toennies He-LiF potential''/                                             &
&                 ''           He-Li A(1) (eV)                     = '',1pe12.5/                   &
&                 ''           He-Li b(1) (1/ang)                  = '',1pe12.5/                   &
&                 ''           He-Li C(6) (meV-angstroms**6)       = '',1pe12.5/                   &
&                 ''           He-F  A(1) (eV)                     = '',1pe12.5/                   &
&                 ''           He-F  b(1) (1/ang)                  = '',1pe12.5/                   &
&                 ''           He-F  C(6) (meV-angstroms**6)       = '',1pe12.5/)') pterm(1),      &
&                 pterm(2),pterm(3),pterm(4),pterm(5),pterm(6)
endif
!
if(.not.brief) write(nout,'(///'' scaling information:  length scale value = '',1pe12.5,'' ang''/  &
&                              ''                       mass scale value   = '',1pe12.5,'' amu''/)') slen,smass
!
!        print out calculation and plot options
!
write(nout,'('' number of lattice planes = '',i5/)') nlayer
write(nout,'(''      z location='',1pe12.5,'' angstroms,  strength coefficient='',e12.5)') (zlayer(n),zcoeff(n),n=1,nlayer)
!
!        compute reciprocal lattice spacing
!
xlatt=twopi/a
ylatt=twopi/b
write(nout,'(//'' x axis lattice constant (ang) = '',1pe12.5/                                      &
&              '' x axis reciprocal lattice spacing (1/ang) = '',1pe12.5/                          &
&              '' y axis lattice constant (ang) = '',1pe12.5)') a,xlatt,b
write(nout,'('' y axis reciprocal lattice spacing (1/ang) = '',1pe12.5/)') ylatt
!
!        convert to cgs units
!
if(ptype.eq.1) then
    pterm(1)=pterm(1)*conve*convl
    pterm(2)=pterm(2)*convl
elseif(ptype.eq.2) then
    pterm(1)=pterm(1)*conve
    pterm(2)=pterm(2)/convl
    pterm(3)=pterm(3)*convl
elseif(ptype.eq.3) then
    pterm(1)=pterm(1)*conve
    pterm(2)=pterm(2)*convl
elseif(ptype.eq.4) then
    pterm(1)=pterm(1)*onee3*conve
    pterm(4)=pterm(4)*onee3*conve
    pterm(2)=pterm(2)/convl
    pterm(5)=pterm(5)/convl
    pterm(3)=pterm(3)*conve*convl**6
    pterm(6)=pterm(6)*conve*convl**6
endif
tmass=tmass*convm
pmass=pmass*convm
a=a*convl
b=b*convl
do 11 n=1,nlayer
    zlayer(n)=zlayer(n)*convl
11 continue
lambda=omega*tmass/hbar
slen=slen*convl
smass=smass*convm
sener=(hbar/slen)**2/smass
smom=hbar/slen
!
!        scale to dimensionless units and print out variables
!
if(ptype.eq.1) then
    pterm(1)=pterm(1)/(sener*slen)
    pterm(2)=pterm(2)/slen
elseif (ptype.eq.2) then
    pterm(1)=pterm(1)/sener
    pterm(2)=pterm(2)*slen
    pterm(3)=pterm(3)/slen
elseif (ptype.eq.3) then
    pterm(1)=pterm(1)/sener
    pterm(2)=pterm(2)/slen
elseif (ptype.eq.4) then
    pterm(1)=pterm(1)/sener
    pterm(4)=pterm(4)/sener
    pterm(2)=pterm(2)*slen
    pterm(5)=pterm(5)*slen
    pterm(3)=pterm(3)/(sener*slen**6)
    pterm(6)=pterm(6)/(sener*slen**6)
endif
tmass=tmass/smass
pmass=pmass/smass
lambda=lambda*slen**2
a=a/slen
b=b/slen
do 23 n=1,nlayer
    zlayer(n)=zlayer(n)/slen
23 continue
pmom=pmom/smom
!
if (.not.polar) then
    numtin=1
    tinin=tin
endif
if (.not.azim) numphi=1
 tinin=tinin*dtr
tinout=tinout*dtr
phiin=phiin*dtr
phiout=phiout*dtr
zmu=pmom**2
tin=tinin
if (numphi.gt.1) delphi=(phiout-phiin)/real(numphi-1,LONGreal)
if (numtin.gt.1) deltin=(tinout-tinin)/real(numtin-1,LONGreal)
numpin=0
do it=1,numtin
    phi=phiin
    do ip=1,numphi
        numpin=numpin+1
        if (numpin.gt.maxpin) then
            write(nout,'('' maxpin too small, abort'')')
            call abort
        endif
        pin(1,numpin)=pmom*sin(tin)*cos(phi)
        pin(2,numpin)=pmom*sin(tin)*sin(phi)
        pin(3,numpin)=pmom*cos(tin)
        phinow(numpin)=phi/dtr
        tinnow(numpin)=tin/dtr
        phi=phi+delphi
    enddo
    tin=tin+deltin
enddo
!
!        compute reciprocal lattice spacing in dimensionless units
!
xlatt=twopi/a
ylatt=twopi/b
if(.not.brief) then
    write(nout,'(/'' dimensionless units: ''/'' tmass='',e12.5,'' pmass='',e12.5)') tmass,pmass
    write(nout,'('' lambda='',e12.5/)') lambda
    write(nout,'('' pmom='',1pe12.5,'' a='',e12.5,'' b='',e12.5)') pmom,a,b
    write(nout,'('' n='',i5,'' px='',1pe12.5,'' py='',e12.5,'' pz='',e12.5,'' phi='',e12.5,        &
&                '' theta='',e12.5)') (n,pin(1,n),pin(2,n),pin(3,n),phinow(n),tinnow(n),n=1,numpin)
    write(nout,'('' pterms='',1p,10e12.5)') (pterm(n),n=1,npterm)
    write(nout,'('' a='',1pe12.5,'' b='',e12.5,'' xlatt='',e12.5,'' ylatt='',e12.5)') a,b,xlatt,ylatt
    write(nout,'(''       zlayer ('',i2,'') = '',1pe12.5)') (n,zlayer(n),n=1,nlayer)
endif
!
!        print out quadrature info
!
if (qtype.eq.1) then
    write(nout,'(/'' Chebyshev quadrature rule used: '')')
    write (nout,'('' number of quadrature points for specular peak integrals = '',i5)') nspec
    write (nout,'('' number of quadrature points for non-specular peak integrals = '',i5)') nnspec
    write (nout,'('' number of quadrature points for virtual peak integrals = '',i5/)') nvirtl
elseif (qtype.eq.2) then
    write(nout,'(/'' Hermite cubic polynomial quadrature rule used: '')')
    write (nout,'('' number of quadrature knots for specular peak integrals = '',i5)') nsknt
    write (nout,'('' number of quadrature knots for non-specular peak integrals = '',i5)') nnsknt
    write (nout,'('' number of quadrature knots for virtual peak integrals = '',i5)') nvknt
endif
!
end subroutine final_input


!DIR$ FREE
!CRAY - Add source form directive
subroutine mkcube (vfactr,xlatt,ylatt,pin,brief,pmass,smom,order,nsknt,nnsknt,nvknt,points,        &
&                  wght,mindex,nindex,nphys,mnindx)
!
!        Copyright (C) 1993 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Calculate partition points and quadrature weights based upon a Hermite
!                     piecewise cubic quadrature method for each physical and virtual Bragg peak
!        CODED:       F90 version coded 1 May 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Quetzal Computational Associates
!                     3200 Carlisle N.E.
!                     Albuquerque, NM   87110-1664
!
    use kind_definitions
    use constants
    use input_output_units
!
    implicit none
!
    integer order,nsknt,nnsknt,nvknt,npnts,nphys
    real (kind=LONGreal), parameter :: zero=0.0,pt5=0.5,one=1.0,two=2.0,ten8=1.e8
    real (kind=LONGreal), dimension(3) :: pin
    real (kind=LONGreal), dimension(order), intent(out) :: points
    complex (kind=LONGreal), dimension(order), intent(out) :: wght
    integer, dimension(order), intent(out)  :: mindex,nindex
    integer, dimension(2,nphys) :: mnindx
    logical :: brief
    integer :: n,m,npeak,norder,nstart,nstop,nsing,kk,nnow,kpnt,mmin,mmax,nmin,nmax
    real (kind=LONGreal) xlatt,ylatt,smom,qzsqr,qx,qy,qz,qzwave,vfactr,pmag,xmin,xmax,ymin,ymax,   &
&                        pextra,pconst,qxwave,twoqz,pmass,psqr
!
!        compute lattice size required for this calculation
!
    write(nout,'(/'' virtual lattice factor = '',1pe12.5//)') vfactr
    psqr=pin(1)**2+pin(2)**2+pin(3)**2
    pmag=vfactr*sqrt(psqr)
    xmin=(-pmag-pin(1))/xlatt
    xmax=( pmag-pin(1))/xlatt
    ymin=(-pmag-pin(2))/ylatt
    ymax=( pmag-pin(2))/ylatt
    mmin=int(xmin)
    if(mmin.gt.zero.and.abs(real(mmin,LONGreal)-xmin).gt.small) mmin=mmin+1
    mmax=int(xmax)
    if(mmax.lt.zero.and.abs(real(mmax,LONGreal)-xmax).gt.small) mmax=mmax-1
    nmin=int(ymin)
    if(nmin.gt.zero.and.abs(real(nmin,LONGreal)-ymin).gt.small) nmin=nmin+1
    nmax=int(ymax)
    if(nmax.lt.zero.and.abs(real(nmax,LONGreal)-ymax).gt.small) nmax=nmax-1
!
!        the virtual lattice must be larger than the physical lattice.
!        catch the special case where only the m=n=0 physical lattice
!        point is present.
!
    if(mmin.eq.0.and.mmax.eq.0.and.nmin.eq.0.and.nmax.eq.0) then
        pextra=(xlatt*real(mmax,LONGreal))**2+(ylatt*real(nmax,LONGreal))**2
        vfactr=one
    else
        pextra=zero
    endif
!
    write(nout,'('' minimum x axis lattice number = '',i5/'' maximum x axis lattice number = '',   &
&                    i5)') mmin,mmax
    write(nout,'('' minimum y axis lattice number = '',i5/'' maximum y axis lattice number = '',   &
&                    i5)') nmin,nmax
!
!        compute n index for each m index
!
    write(nout,'(//'' Bragg reflection points (1/angstroms):''/)')
    norder=0
    npeak=0
    do m=mmin,mmax
        pconst=sqrt(psqr*vfactr**2+pextra-(pin(1)+xlatt*real(m,LONGreal))**2)
        ymin=(-pin(2)-pconst)/ylatt
        ymax=(-pin(2)+pconst)/ylatt
        nmin=int(ymin)
        if(ymin.gt.zero.and.abs(real(nmin,LONGreal)-ymin).gt.small) nmin=nmin+1
        nmax=int(ymax) 
        if(ymax.lt.zero.and.abs(real(nmax,LONGreal)-ymax).gt.small) nmax=nmax-1
!
!        calculate reflection points cooresponding to this lattice
!        point
!
        qx=pin(1)+xlatt*real(m,LONGreal)
        qxwave=qx*smom/(ten8*hbar)
        do n=nmin,nmax  
            qy=pin(2)+ylatt*real(n,LONGreal)
            qzsqr=psqr-qx**2-qy**2
!
!        ******* physically accessible recripocal lattice point ********
!
            if(qzsqr.ge.zero) then
                if (qzsqr.le.small) then
                    write(nout,'(/'' singularity at origin, abort in subroutine mkcube. ''/)')
                    call abort
                endif
                if(m.eq.0.and.n.eq.0) then
                    npnts=nsknt
                else
                    npnts=nnsknt
                endif
                npeak=npeak+1
                mnindx(1,npeak)=m
                mnindx(2,npeak)=n
                qz=sqrt(qzsqr)
                qzwave=qz*smom/(ten8*hbar)
                write(nout,'(/'' Bragg peak at m='',i5,'' n ='',i5,'' wave number='',1pe12.5,      &
&                             '' scaled wave number='',e12.5)') m,n,qzwave,qz
                nnow=2*npnts+1
                if (norder+nnow.gt.order) then
                    write(nout,'(/'' Point vector too long in subroutine mkcube, abort''/          &
&                                 '' You have real problems, fix them!''/                          &
&                                '' The maximum allowed now is '',i10/)') order
!CRAY - Previous line removed ,, from FORMAT
                    call abort
                endif
                nstart=norder+1
                nstop=norder+npnts
                norder=norder+npnts+1
                twoqz=two*qz
                nsing=norder
!
!        calculate knots and weights
!
                call rcubic (2,qzsqr,npnts,points(norder:norder+npnts-1),wght(norder:norder+npnts-1))
                norder=norder+npnts
!
!        add in singular point
!
                points(nsing)=qz
                wght(nsing)=cmplx(zero,-pt5*pi/qz,LONGreal)
!
!        put the singular point in the appropiate location to
!        have increasing points
!
                points(nstart)=points(nsing)
                wght(nstart)=wght(nsing)
                do kk=nsing+2,norder
                    nnow=kk
                    if (points(nsing).lt.points(kk)) go to 40
                enddo
                nnow=nnow+1
40              do kk=nsing,nnow-2
                    points(kk)=points(kk+1)
                    wght(kk)=wght(kk+1)
                enddo
                points(nnow-1)=points(nstart)
                wght(nnow-1)=wght(nstart)
!
!       now copy positive partition points into negative ones
!
                kpnt=nstop+1
                do kk=nstop,nstart,-1
                    kpnt=kpnt+1
                    points(kk)=-points(kpnt)
                    wght(kk)=wght(kpnt)
                enddo
!
                mindex(nstart:norder) = m
                nindex(nstart:norder) = n
!
                if(.not.brief) then
                    write(nout,'(/'' Physical peak partition for m='',i5,'' n='',i5/)') m,n
                    write(nout,'('' index='',i10,'' scaled wave number ='',1pe12.5,                &
&                                '' quadrature weight ='',2e15.5)')(kk,points(kk),wght(kk),        &
&                                kk=nstart,norder)
                endif
            else
!
!        ************** virtual reciprocal lattice points *************
!
                nnow=nvknt
                if (norder+2*nnow-1.gt.order) then
                    write(nout,'(/'' Point vector too long in subroutine mkcube, abort''/          &
&                                '' You have '',''real problems, fix them!''/                      &
&                                '' The maximum allowed now is '',i10/)') order
                    call abort
                endif
                nstart=norder+1
                nstop=nstart+nnow-2
                call rcubic (1,qzsqr,nvknt,points(norder:norder+nvknt-1),wght(norder:norder+nvknt-1))
                norder=norder+nvknt
                kpnt=nstop+1
!
!       load negative points
!
                do kk=nstop,nstart,-1
                    kpnt=kpnt+1
                    points(kk)=-points(kpnt)
                    wght(kk)=wght(kpnt)
               enddo
!
               mindex(nstart:norder) = m
               nindex(nstart:norder) = n
!
                write(nout,'(/'' Virtual peak partition for m='',i5,'' n='',i5/)') m,n
                if(.not.brief) then
                    write(nout,'('' index='',i10,'' scaled wave number ='',1pe12.5,                &
&                                '' quadrature weight='',2e15.5)')(kk,points(kk),wght(kk),         &
&                                kk=nstart,norder)
                endif
            endif
        enddo
    enddo
!
!        multiply weights by twice the projectile mass and then
!        invert them
!
    wght(1:norder)=cmplx(one,zero,LONGreal)/(cmplx(two*pmass,zero,LONGreal)*wght(1:norder))
!
!        abort if npeak is not equal to nphys
!
    if (npeak.ne.nphys) then
       write(nout,'(/'' nphys is not equal to npeak in subroutine mkcube, abort.''/'' npeak='',i5, &
&                    '' nphys=  '',i5/)') npeak,nphys
        call abort
    endif
!
!        abort if norder is not equal to order
!
    if (norder.ne.order) then
        write(nout,'(/'' order is not equal to norder in subroutine mkcube, abort.''/              & 
&                     '' order='',i5,'' norder=  '',i5/)') order,norder
        call abort
    endif
!
end subroutine mkcube


!DIR$ FREE
!CRAY - Add source form directive
subroutine next
!
!        Copyright (C) 1990 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!        PURPOSE:     Parse an ascii file for delimeted fields.  Used
!                     for performing free format input.
!        CODED:       Original version coded 5 January 1985
!                     F90 version coded 14 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Formerly:  Department of Physics and Astronomy,
!                                University of New Mexico
!                     Currently: Quetzal Computational Associates
!                                3200 Carlisle N.E.
!                                Albuquerque, NM   87110-1664
!
!        INPUT:
!
!        nin       integer   Fortran unit number for ascii input
!                            (passed in module free_format_parser_variables)
!
!        icpnt     integer   integer pointer to first character after
!                            the previously delimeted field.  On the
!                            first call to this routine, it is
!                            initialized so as to force a read off file
!                            nin.  (passed in module free_format_parser_variables)
!
!
!        OUTPUT:
!
!        field     char      character*80 blank filled string containing
!                            the next valid substring encountered, left
!                            justified and blank filled.  a valid
!                            substring is any set of contiguous
!                            non-delimeters.  delimeters are defined
!                            in the declarations at the beginning of the
!                            routine.  returned in module free_format_parser_variables
!
!        eoff      logical   true=end of file encountered, no valid sub-
!                            string was found.  false=valid sub-string
!                            found this flag is returned in module
!                            free_format_parser_variables
!
!        card      char      character*80 string containing contents of
!                            last record read.  the contents of this
!                            string should not be changed if subsequent
!                            calls to next are going to be made.  this
!                            string is passed in module free_format_parser_variables
!
!        icpnt     integer   integer pointer to first character after
!                            the last delimeted field.
!                            (passed in module free_format_parser_variables)
!
!
!        EXTERNAL ROUTINES USED: none
!
!        FURTHER EXPLANATION:
!
!        sequentially scan 80 column card images from unit nin for
!        non-delimeter substrings.  output is passed through
!        the module free_format_parser_variables, as explained above.
!        scanning stops at the end of the substring or at an end
!        of file.  if an end of file is encountered, the file is
!        left positioned after the end of file.  If an asterisk is
!        found in column 1 of a card, ignore it (this is an input
!        comment card)
!
!        ***************************************************************
!        Note!  This routine assumes that the input file (unit nin)
!        has already been opened.
!        ***************************************************************
!
use free_format_parser_variables
use input_output_units
!
implicit none
!
integer, parameter :: icend=80
character delim(3)*1
logical :: first
integer ::  i,j,k,istart,iend,kmin,nodel
!
save first
data first /.true./
!
!        icend=length of each record (80=standard 80 column input)
!              note that this can be changed, but it should agree
!              with the length of 'field'.
!        nodel=number of delimeters
!        delim(i)=character*1 delimeters
!        icpnt=pointer to current cimaged position on record. this
!              is initialized by search to the position after the
!              header keyword string which starts the input file.
!
data nodel , delim /3 , ' ' , ',' , '='/
!
!        on first pass, initialize icpnt pointer
!
if (first) then
    icpnt=icend+1
    first=.false.
endif
!
!        if icpnt>icend, read the next record off unit 2 into
!        the string 'card'.  next verify that this is a non-
!        blank card.  if it is blank or an input comment card
!        (asterisk in column 1), skip it and get the next record
!
10 eoff=.false.
if(icpnt.gt.icend) then
  read(nin,'(a)',end=50) card
  if(card(1:icend).eq.' '.or.card(1:1).eq.'*') go to 10
  icpnt=1
endif
!
!        get the next sub-string.  we do this as follows.  we
!        look for the next delimeter.  if it is as the same
!        position as the current pointer, we advance the pointer
!        and try again.  if not, then the pointer is at the beginning
!        of a sub-string and the delimeter is trailing this sub-string.
!        note that we look for all the delimeters possible before
!        taking any action.
!
do i=icpnt,icend
  istart=i
  kmin=0
  do j=1,nodel
    k=index(card(i:icend),delim(j))
!
!        index returns positions relative the beginning of the
!        sub-string.  hence we add in the appropiate off-set to
!        give the index relative to the beginning of the string
!        card, not just the sub-string card(i:icend).
!
    if(k.ne.0) then
      k=k+i-1
      if(kmin.eq.0) then
        kmin=k
      else
        kmin=min(k,kmin)
      endif
    endif
  enddo
!
!        if kmin is not equal to the current pointer position, then
!        it must be pointing at the trailing delimeter of a valid
!        sub-string.
!
  if(kmin.ne.i) then
    if(kmin.gt.0) then
      iend=kmin-1
      go to 40
    endif
!
!        if we fall through, there was no delimeter
!        found on the remainder of this record.  this means
!        the entire remainder of this record is a valid sub-string
!
    iend=icend
    go to 40
  endif
enddo
!
!        if we fall through this loop, there were no more non-
!        delimeters on this record.  go get next record
!
icpnt=icend+1
go to 10
!
!        put the sub-string into the string 'field'.  note
!        that fortran 77 pads the string with blanks
!
40 field=card(istart:iend)
icpnt=iend+2
return
!
!        end of file encountered, set flag and return
!
50 eoff=.true.
icpnt=icend+1
return
end subroutine next


!DIR$ FREE
!CRAY - Add source form directive
subroutine value (result,itype)
!
!        Copyright (C) 1990 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!        PURPOSE:     Parse an ascii file for delimeted fields and
!                     return either a numeric value or an ascii
!                     string.  Used for performing free format input.
!        CODED:       Original Fortran 77 coding: 5 January 1985
!                     F90 version coded 14 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Formerly:  Department of Physics and Astronomy
!                                University of New Mexico
!                     Currently: Quetzal Computational Associates
!                                3200 Carlisle N.E.
!                                Albuquerque, NM   87110-1664
!
!        INPUT:
!
!        icpnt     integer   integer pointer to first character after
!                            the previously delimeted field.  On the
!                            first call to this routine, it is
!                            initialized so as to force a read off file
!                            nin.  (passed in common /image/ )
!
!
!        OUTPUT:
!
!        field     char      character*80 blank filled string containing
!                            the next valid substring encountered, left
!                            justified and blank filled.  a valid
!                            substring is any set of contiguous
!                            non-delimeters.  delimeters are defined
!                            in the declarations at the beginning of the
!                            routine.  returned in common /cimage/.
!
!        eoff      logical   true=end of file encountered, no valid sub-
!                            string was found.  false=valid sub-string
!                            found this flag is returned in common
!                            /image/.
!
!        card      char      character*80 string containing contents of
!                            last record read.  the contents of this
!                            string should not be changed if subsequent
!                            calls to next are going to be made.  this
!                            string is passed in common /cimage/.
!
!        icpnt     integer   integer pointer to first character after
!                            the last delimeted field.
!                            (passed in common /image/ )
!
!        result    real      numerical (or logical) value of the field.
!                            if the field was an integer, result is the
!                            floated value.  if the field was alpha-
!                            numeric, this variable is not set.
!
!        itype     integer   identifier showing type of field read.
!                            -1 = end of file encountered, no data read.
!                            0 = alphanumeric data, no value returned in
!                            result.
!                            1 = integer or real
!
!        EXTERNAL ROUTINES USED: next
!
!        FURTHER EXPLANATION:
!
!        free-format numeric/alphanumeric input.  when this routine is
!        called, it will call subroutine next to fetch the next
!        valid field off unit nin.  this field will then be passed
!        to value in the module free_format_parser_variables as an internal 
!        file.  value then attempts a read on the file. if the field was
!        integer, then value returns a floated result.  in the
!        event no further data is available, the logical variable
!        eoff in the module free_format_parser_variables will be set true.  
!        in all other cases it will be false.  also, itype=-1 will indicate 
!        an end of file was encountered.  finally, the field being processed 
!        is unchanged by value and can be used by other routines since
!        it is passed in the module free_format_parser_variables
!
use kind_definitions
use free_format_parser_variables
!
implicit none
!
real (kind = LONGreal) :: result
integer :: itype
!
!        get next field off unit 2
!
call next
if(eoff) then
    itype=-1
else
!
!        read field as a numeric
!

    if (field(1:1).ge.'a' .and. field(1:1).le.'z') go to 10
    read(field,fmt='(bn,f20.0)',err=10) result
    itype=1
    go to 20
!
!        the only possibility left is that this was an alphanumeric
!
10  itype=0
20  continue
endif
!
end subroutine value


!DIR$ FREE
!CRAY - Add source form directive

subroutine numpk (vfactr,xlatt,ylatt,pin,nphys,nvirt)
!
!
!        Copyright (C) 1990 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Calculate number of of physical and virtual
!                     Bragg peaks
!        CODED:       Serial version coded 1 November 1990.
!                     CM version coded 24 August 1991
!                     F90 version coded 20 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Formerly:  Department of Physics and Astronomy,
!                                University of New Mexico
!                     Currently: Quetzal Computational Associates
!                                3200 Carlisle N.E.
!                                Albuquerque, NM   87110-1664
!
!
!        INPUT:
!
!
!        vfactr    real     multiplier for extending the physical
!                           reciprocal lattice to virtual points
!
!        xlatt     real     reciprocal lattice constant in x direction
!                           (dimensionless)
!
!        ylatt     real     reciprocal lattice constant in y direction
!                           (dimensionless)
!
!        pin       real a.  incoming momentum components (dimensionless)
!
!
!        OUTPUT:
!
!        nphys     integer  Number of physical Bragg peaks
!
!        nvirt     integer  Number of virtual Bragg peaks
!
!        EXTERNAL ROUTINES USED: none
!
use kind_definitions
!
implicit none
!
real (kind = LONGreal), parameter :: zero=0.0,one=1.0,small=1.e-10
real (kind = LONGreal), dimension(3) :: pin
real (kind = LONGreal) :: psqr,pmag,xmin,xmax,ymin,ymax,xlatt,ylatt,pconst,qx,qy,qzsqr,            &
&                         vfactr,pextra
integer :: mmin,mmax,nmin,nmax,nphys,nvirt,m,n
!
!        compute lattice size required for this calculation
!
psqr=dot_product(pin,pin)
pmag=vfactr*sqrt(psqr)
xmin=(-pmag-pin(1))/xlatt
xmax=( pmag-pin(1))/xlatt
ymin=(-pmag-pin(2))/ylatt
ymax=( pmag-pin(2))/ylatt
mmin=int(xmin)
if(mmin.gt.zero.and.abs(real(mmin,LONGreal)-xmin).gt.small) mmin=mmin+1
mmax=int(xmax)
if(mmax.lt.zero.and.abs(real(mmax,LONGreal)-xmax).gt.small) mmax=mmax-1
nmin=int(ymin)
if(nmin.gt.zero.and.abs(real(nmin,LONGreal)-ymin).gt.small) nmin=nmin+1
nmax=int(ymax)
if(nmax.lt.zero.and.abs(real(nmax,LONGreal)-ymax).gt.small) nmax=nmax-1
!
!        the virtual lattice must be larger than the physical lattice.
!        catch the special case where only the m=n=0 physical lattice
!        point is present.
!
if(mmin.eq.0.and.mmax.eq.0.and.nmin.eq.0.and.nmax.eq.0) then
    pextra=(xlatt*real(mmax,LONGreal))**2+(ylatt*real(nmax,LONGreal))**2
    vfactr=one
else
    pextra=zero
endif
!
!        compute n index for each m index
!
nphys=0
nvirt=0
do m=mmin,mmax
    pconst=sqrt(psqr*vfactr**2+pextra-(pin(1)+xlatt*real(m,LONGreal))**2)
    ymin=(-pin(2)-pconst)/ylatt
	  ymax=(-pin(2)+pconst)/ylatt
	  nmin=int(ymin)
	  if(ymin.gt.zero.and.abs(real(nmin,LONGreal)-ymin).gt.small) nmin=nmin+1
	  nmax=int(ymax)
	  if(ymax.lt.zero.and.abs(real(nmax,LONGreal)-ymax).gt.small) nmax=nmax-1
!
!        calculate reflection points cooresponding to this lattice
!        point
!
    qx=pin(1)+xlatt*real(m,LONGreal)
    do n=nmin,nmax
        qy=pin(2)+ylatt*real(n,LONGreal)
        qzsqr=psqr-qx**2-qy**2
!
!        ******* physically accessible recripocal lattice point ********
!
        if(qzsqr.ge.zero) then
            nphys=nphys+1
        else
!
!        ************** virtual reciprocal lattice points *************
!
            nvirt=nvirt+1
        endif
    enddo
enddo
!
end

!DIR$ FREE
!CRAY - Add source form directive
subroutine rcubic (type,a,npnts,xout,wghtout)
!
!
!        Copyright (C) 1993 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Return points and weights for a N-point singular
!                     Hermite cubic quadrature rule
!        CODED:       F90 version coded 1 May 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Quetzal Computational Associates
!                     3200 Carlisle N.E.
!                     Albuquerque, NM   87110-1664
!
!        INPUT:
!
!        type      integer   =1 if a<0
!                            =2 if a>0
!
!        a         real      the point a mentioned above
!
!        npnts     integer   number of points to be used in rule
!
!        big       real      constant passed in common /consts/ which
!                            determines the largest number used in the
!                            code.  In this routine, big is used to
!                            determine when a value is to be considered
!                            infinite.
!
!        nout      integer   Fortran i/o unit used for printed output
!                            (passed in common /iounit/ ).
!
!        x        real a.    real array of length dimensioned
!                            (-1:npnts-1) to be used for computing
!                            knots.  Only x(0:npnts-1) will be returned,
!                            but anything that was in x(-1) on input
!                            will have been trashed.  Be careful!
!
!        wght     complex    complex array of length dimensioned
!                 array      (-1:npnts-1) to be used for computing
!                            weights.  Only wght(0:npnts-1) will be
!                            returned, but anything that was in wght(-1)
!                            on input will have been trashed.  Be
!                            careful!
!
!
!        OUTPUT:
!
!        x        real a.    real array of length dimensioned
!                            (-1:npnts-1) containing quadrature points
!                            (knots) in locations (0:npnts-1)
!
!        wght      complex   complex array dimensioned (-1:npnts-1)
!                  array     containing quadrature weights corresponding
!                            to the points in the x array in locations
!                            (0:npnts-1)
!
!        EXTERNAL ROUTINES USED: abort
!
    use kind_definitions
    use constants
    use input_output_units
!
    implicit none
!
    real (kind=LONGreal), parameter :: zero=0.0,pt5=0.5,one=1.0,onept5=1.5,two=2.0,three=3.0,six=6.0
    real (kind=LONGreal) :: a,a0,b0,d0,e0,a1,b1,d1,e1,a0hat,b0hat,d0hat,e0hat,a1hat,b1hat,d1hat,   &
&                           e1hat,h1,h2,h3,vi0,vi1,vi2,vi3,pi0,pi1,pi2,pi3,i0v,i1v,i0hatv,i1hatv,  &
&                           i0p,i1p,i0hatp,i1hatp,p0p,p1p,p0hatp,p1hatp,aabs
    integer :: npnts,type,n
    complex (kind=LONGreal), dimension(-1:npnts-1) :: wght
    complex (kind=LONGreal) coeff
    complex (kind=LONGreal), dimension(0:npnts-1), intent(out) :: wghtout
    real (kind=LONGreal), dimension(-1:npnts-1) :: x
    real (kind=LONGreal), dimension(0:npnts-1), intent(out) :: xout
!
    external abort
!
!        cubic polynomial coefficient functions
!
    a0(n)=two/(x(n+1)-x(n))**3
    b0(n)=-three/(x(n+1)-x(n))**3*(x(n)+x(n+1))
    d0(n)=six/(x(n+1)-x(n))**3*x(n)*x(n+1)
    e0(n)=x(n+1)**2*(x(n+1)-three*x(n))/(x(n+1)-x(n))**3
    a1(n)=-two/(x(n+1)-x(n))**3
    b1(n)=three/(x(n+1)-x(n))**3*(x(n)+x(n+1))
    d1(n)=-six/(x(n+1)-x(n))**3*x(n)*x(n+1)
    e1(n)=-x(n)**2*(x(n)-three*x(n+1))/(x(n+1)-x(n))**3
    a0hat(n)=one/(x(n+1)-x(n))**2
    b0hat(n)=-(x(n)+two*x(n+1))/(x(n+1)-x(n))**2
    d0hat(n)=x(n+1)*(two*x(n)+x(n+1))/(x(n+1)-x(n))**2
    e0hat(n)=-x(n)*x(n+1)**2/(x(n+1)-x(n))**2
    a1hat(n)=one/(x(n+1)-x(n))**2
    b1hat(n)=-(two*x(n)+x(n+1))/(x(n+1)-x(n))**2
    d1hat(n)=x(n)*(x(n)+two*x(n+1))/(x(n+1)-x(n))**2
    e1hat(n)=-x(n)**2*x(n+1)/(x(n+1)-x(n))**2
!
!        centered difference coefficients
!
    h1(n)=(x(n)-x(n-1))/((x(n+1)-x(n))*(x(n+1)-x(n-1)))
    h2(n)=((x(n+1)-x(n))**2-(x(n)-x(n-1))**2)/((x(n)-x(n-1))*(x(n+1)-x(n))*(x(n+1)-x(n-1)))
    h3(n)=-(x(n+1)-x(n))/((x(n)-x(n-1))*(x(n+1)-x(n-1)))
!
!        integrals needed for virtual points
!
    vi0(n)=asin(x(n+1))-asin(x(n))
    vi1(n)=sqrt(one-x(n)**2)-sqrt(one-x(n+1)**2)
    vi2(n)=pt5*(x(n)*sqrt(one-x(n)**2)-x(n+1)*sqrt(one-x(n+1)**2)+asin(x(n+1))-asin(x(n)))
    vi3(n)=((one-x(n+1))**onept5-(one-x(n))**onept5)/three+sqrt(one-x(n)**2)-sqrt(one-x(n+1)**2)
!
!        integrals needed for physical points
!
    pi0(n)=log((x(n+1)*(one+sqrt(one-x(n)**2)))/(x(n)*(one+sqrt(one-x(n+1)**2))))
    pi1(n)=asin(x(n+1))-asin(x(n))
    pi2(n)=sqrt(one-x(n)**2)-sqrt(one-x(n+1)**2)
    pi3(n)=pt5*(x(n)*sqrt(one-x(n)**2)-x(n+1)*sqrt(one-x(n+1)**2)+asin(x(n+1))-asin(x(n)))
!
!        virtual integrals
!
    i0v(n)=a0(n)*vi3(n)+b0(n)*vi2(n)+d0(n)*vi1(n)+e0(n)*vi0(n)
    i1v(n)=a1(n)*vi3(n)+b1(n)*vi2(n)+d1(n)*vi1(n)+e1(n)*vi0(n)
    i0hatv(n)=a0hat(n)*vi3(n)+b0hat(n)*vi2(n)+d0hat(n)*vi1(n)+e0hat(n)*vi0(n)
    i1hatv(n)=a1hat(n)*vi3(n)+b1hat(n)*vi2(n)+d1hat(n)*vi1(n)+e1hat(n)*vi0(n)
!
!        physical non-principal part integrals
!
    i0p(n)=a0(n)*pi3(n)+b0(n)*pi2(n)+d0(n)*pi1(n)+e0(n)*pi0(n)
    i1p(n)=a1(n)*pi3(n)+b1(n)*pi2(n)+d1(n)*pi1(n)+e1(n)*pi0(n)
    i0hatp(n)=a0hat(n)*pi3(n)+b0hat(n)*pi2(n)+d0hat(n)*pi1(n)+e0hat(n)*pi0(n)
    i1hatp(n)=a1hat(n)*pi3(n)+b1hat(n)*pi2(n)+d1hat(n)*pi1(n)+e1hat(n)*pi0(n)
!        physical principal part integrals
!
    p0p(n)=a0(n)*pi3(n)+d0(n)*pi1(n)
    p1p(n)=a1(n)*pi3(n)+d1(n)*pi1(n)
    p0hatp(n)=a0hat(n)*pi3(n)+d0hat(n)*pi1(n)
    p1hatp(n)=a1hat(n)*pi3(n)+d1hat(n)*pi1(n)
!
!        make sure a.gt.0 and finite if type 2.
!
    if (type.eq.2) then
        if (a.le.zero.or.a.ge.big) then
            write(nout,'(/'' a is negative or infinite for type 2 in subroutine rcheby, abort.''/  &
&                        '' a='',1pe12.5/)') a
            call abort
        endif
!
!        make sure a.lt.0 if type = 1
!
    elseif (type.eq.1) then
        if (a.ge.zero) then
            write(nout,'(/'' a is positive for type 1 in subroutine rcheby, abort.''/'' a='',1pe12.5/)') a
            call abort
        endif
!
!        unrecognized type, abort
!
    else
        write(nout,'(/'' Illegal type is subroutine rcheby, abort.''/'' The type was '',i5/)') type
        call abort
    endif
!
!        generate points in the [-1,1] interval
!
    aabs=abs(a)
    call mkknot(1,npnts,x(0:npnts-1),aabs)
!
!        generate symmetric point around x(0)
!
    x(-1)=two*x(0)-x(1)
!
!        zero weights
!
    do n=-1,npnts-1
        wght(n)=cmplx(zero,zero,LONGreal)
    enddo
    coeff=cmplx(-pt5/sqrt(aabs),zero,LONGreal)
!
!***********************************************************************
!        type = 1 (a < 0)
!***********************************************************************
!
    if (type.eq.1) then
        do n=0,npnts-2
            wght(n-1)=wght(n-1)+coeff*cmplx(h3(n)*i0hatv(n),zero,LONGreal)
            wght(n)=wght(n)+coeff*cmplx(i0v(n)+h2(n)*i0hatv(n)+h3(n+1)*i1hatv(n),zero,LONGreal)
            wght(n+1)=wght(n+1)+coeff*cmplx(h1(n)*i0hatv(n)+i1v(n)+h2(n+1)*i1hatv(n),zero,LONGreal)
            if (n+2.ne.npnts) wght(n+2)=wght(n+2)+coeff*cmplx(h1(n+1)*i1hatv(n),zero,LONGreal)
        enddo
        n=npnts-1
        wght(n-2)=wght(n-2)+coeff*cmplx(h3(n-1)*i0hatv(n-1),zero,LONGreal)
        wght(n-1)=wght(n-1)+coeff*cmplx(i0v(n-1)+h2(n-1)*i0hatv(n-1),zero,LONGreal)
!
!***********************************************************************
!        type = 2 (a>0)
!***********************************************************************
!
    elseif (type.eq.2) then
        do n=0,npnts-2
            if(x(n+1).lt.zero.or.x(n).gt.zero) then
!
!        singularity not in this interval
!
                wght(n-1)=wght(n-1)+coeff*cmplx(h3(n)*i0hatp(n),zero,LONGreal)
                wght(n)=wght(n)+coeff*cmplx(i0p(n)+h2(n)*i0hatp(n)+h3(n+1)*i1hatp(n),zero,LONGreal)
                wght(n+1)=wght(n+1)+coeff*cmplx(h1(n)*i0hatp(n)+i1p(n)+h2(n+1)*i1hatp(n),zero,LONGreal)
                if (n+2.ne.npnts) wght(n+2)=wght(n+2)+coeff*cmplx(h1(n+1)*i1hatp(n),zero,LONGreal)
            else
!
!        singularity in this interval
!
                wght(n-1)=wght(n-1)+coeff*cmplx(h3(n)*p0hatp(n),zero,LONGreal)
                wght(n)=wght(n)+coeff*cmplx(p0p(n)+h2(n)*p0hatp(n)+h3(n+1)*p1hatp(n),zero,LONGreal)
                wght(n+1)=wght(n+1)+coeff*cmplx(h1(n)*p0hatp(n)+p1p(n)+h2(n+1)*p1hatp(n),zero,LONGreal)
                if (n+2.ne.npnts) wght(n+2)=wght(n+2)+coeff*cmplx(h1(n+1)*p1hatp(n),zero,LONGreal)
            endif
        enddo
        n=npnts-1
        wght(n-2)=wght(n-2)+coeff*cmplx(h3(n-1)*i0hatp(n-1),zero,LONGreal)
        wght(n-1)=wght(n-1)+coeff*cmplx(i0p(n-1)+h2(n-1)*i0hatp(n-1),zero,LONGreal)
    endif
!
!        map points from the [-1,1] interval into the [0,infinity)
!        interval
!
    call mkknot(2,npnts,x(0:npnts-1),aabs)
    x(-1)=-x(1)
!
!         add the weight for x(-1) into the weight for x(1) and
!         double the weight at the origin
!
    wght(1)=wght(1)+wght(-1)
    wght(0)=cmplx(two,zero,LONGreal)*wght(0)
!
!        map into the xout and wghtout arrays now
!
    xout(0:npnts-1:1) = x(0:npnts-1:1)
    wghtout(0:npnts-1:1) = wght(0:npnts-1:1)
!
end subroutine rcubic


!DIR$ FREE
!CRAY - Add source form directive
subroutine mkknot (type,npnts,x,a)
!
!
!        Copyright (C) 1993 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Calculate knots for a N-point singular
!                     Hermite cubic quadrature rule
!        CODED:       F90 version coded 1 May 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Quetzal Computational Associates
!                     3200 Carlisle N.E.
!                     Albuquerque, NM   87110-1664
!
!
!        INPUT:
!
!        type      integer   =1 returns knots in [-1,1]
!                            =2 maps knots in [-1,1] into knots in
!                               [0,infinity)
!
!        a         real      absolute value of a used in rcubic
!
!        npnts     integer   number of points to be used in rule
!
!        x         real a.   real array dimensioned (0:npnts-1)
!                            containing quadrature points (knots)
!                            in the [-1,1] interval.  Only used if
!                            type=2.
!
!        pi        real      value of pi, passed in common /consts/
!
!        nout      integer   Fortran i/o unit used for printed output
!                            (passed in common /iounit/ ).
!
!
!        OUTPUT:
!
!        x         real a.   real array dimensioned (0:npnts-1)
!                            containing quadrature points (knots)
!                            mapped into appropiate interval
!
!        EXTERNAL ROUTINES USED: abort
!
    use kind_definitions
    use constants
    use input_output_units
!
    implicit none
!
    integer npnts,n,type,nzero
    real (kind=LONGreal) a,ratio,pt5pi,delta,theta
    real (kind=LONGreal), dimension(0:npnts-1), intent(inout) :: x
    real (kind=LONGreal), parameter :: zero=0.0,pt5=0.5,one=1.0,two=2.0
!
    external abort
!
!***********************************************************************
!        type=1 -- generate points in the [-1,1] interval
!***********************************************************************
!
    if (type.eq.1) then
!
!        generate points located at Chebyshev quadrature locations
!
        x(0)=-one
        ratio=pt5/real(npnts-1,LONGreal)
        pt5pi=pt5*pi
        do n=1,npnts-1
            theta=ratio*(two*real(n,LONGreal)-one)*pi
            x(npnts-n)=cos(theta)
        enddo
!
!        make sure there is not a points at the origin
!
        do n=0,npnts-1
            if(abs(x(n)).lt.small) then
                write(nout,'(/'' knot has been placed at the origin, abort in mkknot.''/)')
                call abort
            endif
        enddo
!
!        make sure the points on either side of the origin are
!        equally spaced with respect to it.
!
       nzero=-1
       do n=0,npnts-2
           if(x(n).lt.zero.and.x(n+1).gt.zero) nzero=n
       enddo
       if (nzero.le.0.or.nzero.ge.npnts-2) then
           write(nout,'(/'' bad placement of knots relative to the singularity, abort in mkknot.''/)')
           call abort
       endif
       delta=min(abs(x(nzero)),abs(x(nzero+1)))
       x(nzero)=-delta
       x(nzero+1)=delta
    else
!
!***********************************************************************
!        type=2 -- gmap points in the [-1,1] interval into [0,infinity)
!***********************************************************************
!
        x(0)=zero
        x(1:npnts-1)=sqrt(a*(one+x(1:npnts-1))/(one-x(1:npnts-1)))
    endif
!
end subroutine mkknot
 


!DIR$ FREE
!CRAY - Add source form directive
subroutine error (card,field,ier)
!
!        Copyright (C) 1991 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Graceful free-format error routine
!        CODED:       CM version coded 24 August 1991
!                     F90 version coded 14 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Formerly:  Department of Physics and Astronomy,
!                                University of New Mexico
!                     Currently: Quetzal Computational Associates
!                                3200 Carlisle N.E.
!                                Albuquerque, NM   87110-1664
use input_output_units
!
implicit none
!
character card*80,field*80
integer ier
!
if(ier.eq.-1) then
    write(nout,'('' unexpected eof while reading input, abort'')')
    write(nout,'(/'' was reading the input line: ''/1x,a80)') card
elseif (ier.eq.0) then
    write(nout,'('' expected a number on input and instead encountered the word: ''/1x,a80/'' abort''/)') field
    write(nout,'(/'' was reading the input line: ''/1x,a80)') card
endif
!
close (unit=nin)
close (unit=nout)
!
call abort
!
end subroutine error


!DIR$ FREE
!CRAY - Add source form directive
subroutine bterm1 (mindex,nindex,q,p,lambda,xlatt,ylatt,nlayer,zlayer,zcoeff,term,                 &
&                 poten,pterm,npterm,bounce)
!
!
!        Copyright (C) 1991 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     calculate the B  (q,p) term for a single momentum
!                                    mn
!        CODED:       CM version coded August 1991
!                     F90 version coded 14 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Formerly:  Department of Physics and Astronomy,
!                                University of New Mexico
!                     Currently: Quetzal Computational Associates
!                                3200 Carlisle N.E.
!                                Albuquerque, NM   87110-1664
!
!
use kind_definitions
use constants
use input_output_units
!
implicit none
!
integer :: npterm,nlayer
real (kind=LONGreal), parameter :: zero=0.0,one=1.0,onept5=1.5,two=2.0,four=4.0,eight=8.0,         &
&                                  pt25=0.25,pt5=0.5,three=3.0,three60=360.0,five=5.0,             &
&                                  twenty_four=24.0,six=6.0,seven=7.0,forty_eight=48.0,            &
&                                  thirteen=13.0,seventy_two=72.0,twenty_five=25.0,                &
&                                  two_hundred_eighty_eight=288.0,one_hundred_twenty=120.0,        &
&                                  seven_hundred_twenty=720.0
logical bounce
real (kind=LONGreal) :: work1,work2,q,p
real (kind=LONGreal), dimension(nlayer) :: zlayer,zcoeff
real (kind=LONGreal), dimension(npterm) :: pterm
real (kind=LONGreal) :: xlatt,ylatt,sigsq,expon,const,const2,rlamda,coeff,xreal,yreal,const1,      &
&                       ali,af,bli,bf,cli,cf,lambda
complex (kind=LONGreal) :: term,ccoeff,ecoeff,literm,fterm,sum,zf,zli,log_zfm1,log_zlim1,arg,      &
&                          zconst1,zconst2,zconst3,zconst4,zconst5,zconst6,zconst7,zconst8,        &
&                          zconst9,zconst10,zconst11
integer :: poten,nl,mindex,nindex
!
coeff=(xlatt*ylatt)/twopi**onept5
rlamda=one/(four*lambda)
!
!        calculate delta vector magnitude and store in work array
!
work1=sqrt((xlatt*real(mindex,LONGreal))**2+(ylatt*real(nindex,LONGreal))**2+(q-p)**2)
work2=q-p
!
!        calculate Fourier transform of potential
!
if (poten.eq.1) then
!
!        Yukawa potential Fourier transform calculation
!
    const=sqrt(two/pi)*pterm(1)
    sigsq=one/pterm(2)**2
    term=cmplx(const/(sigsq+work1**2),zero,LONGreal)
elseif (poten.eq.2) then
!
!        Morse potential Fourier transform calculation
!
    expon=exp(pterm(2)*pterm(3))
    const=pterm(1)*sqrt(two/pi)*expon
    const2=two*sqrt(two/pi)*pterm(1)*expon
    if ( work1.gt.small ) then
        term=cmplx(const/work1*(expon*sin(two*atan(work1/(two*pterm(2))))/four*pterm(2)**2+       &
&                work1**2-two*sin(two*atan(work1/pterm(2)))/pterm(2)**2+work1**2),zero,LONGreal)
    else
        term=cmplx(const2*(expon/(eight*pterm(2)**3)-two/pterm(2)**3),zero,LONGreal)
    endif
elseif (poten.eq.3) then
!
!        repulsive square barrier potential
!
    const=sqrt(two/pi)*pterm(1)
    if ( abs(work2).gt.small ) then
        term=cmplx(const/work2*sin(pt5*pterm(2)*work2),zero,LONGreal)
    else
        term=cmplx(pt5*const*pterm(2),zero,LONGreal)
    endif
    coeff=one
elseif (poten.eq.4) then
!
!        Tang/Toennies Hi-LiF unit cell potential
!
    ali=pterm(1)
    af=pterm(4)
    bli=pterm(2)
    bf=pterm(5)
    cli=pterm(3)
    cf=pterm(6)
    xreal=twopi/xlatt
    yreal=twopi/ylatt
    if (abs(xreal-yreal).gt.small) then
        write(nout,'(/'' LiF lattice constants are not the same, abort in bterm''/)')
        call abort
    endif
    const1=sqrt(two/pi)
    const2=(two)**onept5/sqrt(pi)
    arg=cmplx(zero,-xreal/sqrt(two),LONGreal)
    zconst1=cmplx(-one/twenty_four,zero,LONGreal)
    zconst2=cmplx(one/six,zero,LONGreal)
    zconst3=cmplx(-pt25,zero,LONGreal)
    zconst4=zconst2
    zconst5=zconst1
    zconst6=zconst1
    zconst7=cmplx(seven/forty_eight,zero,LONGreal)
    zconst8=cmplx(-thirteen/seventy_two,zero,LONGreal)
    zconst9=cmplx(twenty_five/two_hundred_eighty_eight,zero,LONGreal)
    zconst10=cmplx(-one/one_hundred_twenty,zero,LONGreal)
    zconst11=cmplx(-one/seven_hundred_twenty,zero,LONGreal)
!
!        zero results
!
    term=cmplx(zero,zero,LONGreal)
!
!        calculate unit surface cell coefficients
!
    if ( mod(mindex+nindex,2).eq.0 ) then
        ccoeff=cmplx(one,zero,LONGreal)
    else
        ccoeff=-cmplx(one,zero,LONGreal)
    endif
!
!        calculate l=2 coefficients
!
    ecoeff=exp(arg*cmplx(work2,zero,LONGreal))
!
!        calculate z (needded for attractive term)
!
    zf=cmplx(one,zero,LONGreal)/cmplx(one,-work1/bf,LONGreal)
    zli=cmplx(one,zero,LONGreal)/cmplx(one,-work1/bli,LONGreal)
!
!        do repulsive term first
!
!
!        non-zero momentum case
!
    if (work1.gt.small) then
        literm=cmplx(const1*ali/(work1*(work1**2+bli**2))*sin(two*atan(work1/bli)),zero,LONGreal)
        fterm=cmplx(const1*af/(work1*(work1**2+bf**2))*sin(two*atan(work1/bf)),zero,LONGreal)
!
!        zero momentum case
!
    else
        literm=cmplx(const2*(ali/bli**3),zero,LONGreal)
        fterm=cmplx(const2*(af/bf**3),zero,LONGreal)
    endif
!
!
!        odd numbered layer
!
    term=term+literm+ccoeff*fterm
!
!        even numbered layer
!
    term=term+(literm*ccoeff+fterm)*ecoeff
!
!        calculate ln(1-z) for use in attractive part calculation
!
    if (work1.gt.small) then
        log_zfm1=log(cmplx(one,zero,LONGreal)-zf)
        log_zlim1=log(cmplx(one,zero,LONGreal)-zli)
    endif
!
!        evaluate attractive part second
!
!
!        non-zero momentum case
!
    if ( work1.gt.small ) then
        literm=cmplx(const1*cli*bli**four/work1*aimag((log_zlim1*(zconst1+zli*zconst2+            &
&                  zli**two*zconst3+zli**three*zconst4+zli**four*zconst5)+zli*(zconst6+            &
&                  zli*zconst7+zli**two*zconst8+zli**three*zconst9+zli**four*zconst10+             &
&                  zli**five*zconst11))/zli**four),zero,LONGreal)
        fterm=cmplx(const1*cf*bf**four/work1*aimag((log_zfm1*(zconst1+zf*zconst2+                 &
&                  zf**two*zconst3+zf**three*zconst4+zf**four*zconst5)+zf*(zconst6+                &
&                  zf*zconst7+zf**two*zconst8+zf**three*zconst9+zf**four*zconst10+                 &
&                  zf**five*zconst11))/zf**four),zero,LONGreal)
!
!        zero momentum case
!
    else
        literm=cmplx((const1*cli*bli**three)/three60,zero,LONGreal)
        fterm=cmplx((const1*cf*bf**three)/three60,zero,LONGreal)
    endif
!
!        odd numbered layer
!
    term=term-(literm+ccoeff*fterm)
!
!        even numbered layer
!
    term=term-(literm*ccoeff+fterm)*ecoeff
!
else
    write(nout,'(/'' unsupported potential in bterm, abort''/'' requested potential = '',i3/)') poten
    call abort
endif
!
!        put in phase factor associated with the position of the layers
!
if(nlayer.gt.1) then
    sum=cmplx(zero,zero,LONGreal)
!
    do nl=1,nlayer
        sum=sum+zcoeff(nl)*exp(cmplx(zero,work2*zlayer(nl),LONGreal))
    enddo
!
    term=term*sum
else
    term=term*exp(cmplx(zero,work2*zlayer(1),LONGreal))
endif
!
!        finish calculation
!
if(.not.bounce) then
    term=coeff*term
else
    term=coeff*term*exp(-rlamda*work1**2)
endif
!
!
end subroutine bterm1



!DIR$ FREE
!CRAY - Add source form directive
subroutine doplot (morder,order,matrix)
!
!        Copyright (C) 1993 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Make a printer plot of the matrix to be solved.
!        CODED:       F90 version coded 2 May 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Quetzal Computational Associates
!                     3200 Carlisle N.E.
!                     Albuquerque, NM   87110-1664
!
!        INPUT:
!
!
!        cdisk     integer   unit number for matrix to plot (passed
!                            via common /iounit/)
!
!        order     integer   order of matrix
!
!        matrix     complex   complex scratch array of length order
!                  array
!
!        mxplot    integer   order of matrix plot
!
!        icont     integer   integer array dimensioned (mxplot,mxplot)
!                  array     used for plotting matrix
!
!        cont      real      real array dimensioned (mxplot,mxplot)
!                  array     used for plotting matrix
!
!        map       char      character*1 array dimensioned
!                  array     (mxplot,mxplot) used for plotting matrix
!
!        nout      integer   Fortran unit number for printed output
!                            (passed in common /iounit/ )
!
!        small     real      Small number whose inverse is used as a
!                            very large number (passed in common
!                            /consts/ )
!
!
    use kind_definitions
    use constants
    use input_output_units
!
    implicit none
!
    real (kind=LONGreal), parameter :: zero=0.0,one=1.0,ten4=1.e4,ten8=1.e8
    integer, parameter :: mxplot=70
    integer :: order,morder
    complex (kind=LONGreal), dimension(morder,morder) :: matrix
    character map(mxplot,mxplot)*1
    real (kind=LONGreal), dimension(mxplot,mxplot) :: cont
    integer, dimension (mxplot,mxplot) :: icont
    integer i,j,irmin,irmax,idr,nmax,inow,nn,ijmax,iplot,jplot
    real (kind=LONGreal) scalei,scalej,vmax,vmin,dr,rmin,rmax,cmax,cmin,rnow
!
    cont = zero
    icont = 0
!
    scalei=real(mxplot-3,LONGreal)/real(order-1,LONGreal)
    scalej=real(mxplot-3,LONGreal)/real(order-1,LONGreal)
    vmax=-big
    vmin=big
    do j=1,order
        do i=1,order
            iplot=int(real(i-1,LONGreal)*scalei)+2
            jplot=int(real(j-1,LONGreal)*scalej)+2
            iplot=max(2,min(iplot,mxplot-1))
            jplot=max(2,min(jplot,mxplot-1))
            icont(iplot,jplot)=icont(iplot,jplot)+1
            cont(iplot,jplot)=cont(iplot,jplot)+abs(matrix(i,j))
            vmax=max(vmax,abs(matrix(i,j)))
            vmin=min(vmin,abs(matrix(i,j)))
         enddo
    enddo
    cmax=-ten8
    do j=1,mxplot
        do i=1,mxplot
            cont(i,j)=cont(i,j)/real(max(icont(i,j),1),LONGreal)
            cmax=max(cmax,cont(i,j))
        enddo
    enddo
    cmin=cmax/ten4
    write(nout,'('' Matrix is of order '',i5,'' by '',i5)') order,order
    write(nout,'('' Maximum absolute value in matrix = '',1pe12.5/'' Minimum absolute value in '', &
&                                                                '' matrix = '',1pe12.5/)')vmax,vmin
!
!        clear map
!
    map = ' '
!
    do j=1,mxplot
        do i=1,mxplot
            if((i.eq.1.or.i.eq.mxplot).and.(j.le.5.or.j.ge.mxplot-4)) map(i,j)='*'
            if(j.eq.1.or.j.eq.mxplot) map(i,j)='*'
        enddo
    enddo
!
!        convert to orders of magnitude
!
    irmin=99999
    irmax=-irmin
    do j=2,mxplot-1
        do i=2,mxplot-1
            if(icont(i,j).ne.0) then
                 if(cont(i,j).gt.cmin) then
                    cont(i,j)=log10(cont(i,j))
                    icont(i,j)=int(cont(i,j))
                    irmin=min(irmin,icont(i,j))
                    irmax=max(irmax,icont(i,j))
                else
                    icont(i,j)=0
                endif
            endif
        enddo
    enddo
!
!        calculate plot scale
!
    idr=(irmax-irmin)/9
    if(9*idr.ne.irmax-irmin) idr=idr+1
    write(nout,'('' contour values: '')')
    if(idr.ne.0) then
        nmax=(irmax-irmin)/idr
    else
        nmax=0
    endif
    if(idr*nmax.ne.irmax-irmin) nmax=nmax+1
    nmax=nmax+1
    inow=irmin+nmax*idr
    do nn=nmax,0,-1
        rnow=10.0**real(inow,LONGreal)
        write(nout,'(''     value = '',1pe15.5,'' map value = '',i1)') rnow,nn
        inow=inow-idr
    enddo
!
!        build map
!
    dr=0.5*real(idr,LONGreal)
    ijmax=mxplot-1
    do j=2,ijmax
        do i=2,ijmax
            if(icont(i,j).ne.0) then
                inow=irmin+nmax*idr
                rmin=real(inow,LONGreal)-dr
                do nn=nmax,0,-1
                    rmax=rmin
                    rmin=real(inow,LONGreal)-dr
                    if(nn.eq.nmax) rmax=max(rmax,big)
                    if(cont(i,j).ge.rmin.and.cont(i,j).le.rmax) then
                        map(i,j)=char(48+nn)
                        exit
                    endif
                    inow=inow-idr
                enddo
            endif
        enddo
    enddo
!
!        print map
!
    write(nout,'(//)')
    do i=1,mxplot
        write(nout,'(1x,132a1)') (map(i,j),j=1,mxplot)
    enddo
!
end subroutine doplot


!DIR$ FREE
!CRAY - Add source form directive
subroutine build (morder,order,points,wght,mindex,nindex,lambda,xlatt,ylatt,nlayer,zlayer,zcoeff,  &
&                 matrix,ptype,pterm,npterm,bounce)
!
!        Copyright (C) 1993 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Build the matrix to be solved.
!        CODED:       F90 version coded 2 May 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Quetzal Computational Associates
!                     3200 Carlisle N.E.
!                     Albuquerque, NM   87110-1664
!
    use kind_definitions
!
    implicit none
    integer :: morder,order,ptype,nlayer,npterm,i,j
    real (kind=LONGreal), dimension(morder) :: points
    real (kind=LONGreal), dimension(nlayer) :: zlayer,zcoeff
    real (kind=LONGreal), dimension(npterm) :: pterm
    real (kind=LONGreal) lambda,xlatt,ylatt
    real (kind=LONGreal), dimension(order) :: work
    complex (kind=LONGreal), dimension(morder,morder) :: matrix
    complex (kind=LONGreal), dimension(morder) :: wght
    logical bounce
    integer, dimension(morder) :: mindex,nindex
    integer, dimension(order) :: mtmp,ntmp
!
!        loop over entire matrix and build matrix terms
!
    do j=1,order
        do i=1,order
            work(i) = points(j)
            mtmp(i) = mindex(i) - mindex(j)
            ntmp(i) = nindex(i) - nindex(j)
        enddo
!
!        calculate terms for this column
!
        call bterm (mtmp(1:order),ntmp(1:order),points(1:order),work(1:order),order,lambda,        &
&                   xlatt,ylatt,nlayer,zlayer,zcoeff,matrix(1:order,j),ptype,pterm,npterm,bounce)
!
    enddo
!
!        flip sign of matrix elements and add diagonal term
!
    matrix(1:order:1,1:order:1)=-matrix(1:order:1,1:order:1)
    do i=1,order
        matrix(i,i)=wght(i)+matrix(i,i)
    enddo
!
end subroutine build



!DIR$ FREE
!CRAY - Add source form directive
subroutine mkchev (vfactr,xlatt,ylatt,pin,brief,pmass,smom,order,nspec,nnspec,nvirtl,points,wght,  &
&                  mindex,nindex,nphys,mnindx)
!
!
!        Copyright (C) 1990 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!
!        PURPOSE:     Calculate parititon points and quadrature
!                     weights based upon a Chebyshev quadrature
!                     method for each physical and virtual Bragg peak
!        CODED:       Serial version coded 5 December 1990.
!                     CM version coded 24 August 1991
!                     F90 version coded 20 March 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Formerly:  Department of Physics and Astronomy,
!                                University of New Mexico
!                     Currently: Quetzal Computational Associates
!                                3200 Carlisle N.E.
!                                Albuquerque, NM   87110-1664
!
!
use kind_definitions
use constants
use input_output_units
!
implicit none
!
integer order,nphys
real (kind = LONGreal), parameter :: zero=0.0,pt5=0.5,one=1.0,two=2.0,ten8=1.e8
real (kind = LONGreal), dimension(3) :: pin
real (kind = LONGreal), dimension(order), intent(out)  :: points
complex (kind = LONGreal), dimension(order), intent(out)  :: wght
integer, dimension(order), intent(out)  :: mindex,nindex
integer, dimension(2,nphys) :: mnindx
integer :: nspec,nnspec,nvirtl,npnts,mmin,mmax,nmin,nmax,norder,n,npeak,nnow,nstart,kpnt,nsing,kk, &
&       nstop,m
logical :: brief
real (kind = LONGreal) :: pmass,vfactr,xmin,xmax,ymin,ymax,xlatt,ylatt,pextra,pconst,qxwave,       &
&                         qzsqr,qx,qy,smom,twoqz,psqr,pmag,qz,qzwave
!
!
!        compute lattice size required for this calculation
!
write(nout,'(/'' virtual lattice factor = '',1pe12.5//)') vfactr
psqr=dot_product(pin,pin)
pmag=vfactr*sqrt(psqr)
xmin=(-pmag-pin(1))/xlatt
xmax=( pmag-pin(1))/xlatt
ymin=(-pmag-pin(2))/ylatt
ymax=( pmag-pin(2))/ylatt
mmin=int(xmin)
if(mmin.gt.zero.and.abs(real(mmin,LONGreal)-xmin).gt.small) mmin=mmin+1
mmax=int(xmax)
if(mmax.lt.zero.and.abs(real(mmax,LONGreal)-xmax).gt.small) mmax=mmax-1
nmin=int(ymin)
if(nmin.gt.zero.and.abs(real(nmin,LONGreal)-ymin).gt.small) nmin=nmin+1
nmax=int(ymax)
if(nmax.lt.zero.and.abs(real(nmax,LONGreal)-ymax).gt.small) nmax=nmax-1
!
!        the virtual lattice must be larger than the physical lattice.
!        catch the special case where only the m=n=0 physical lattice
!        point is present.
!
if(mmin.eq.0.and.mmax.eq.0.and.nmin.eq.0.and.nmax.eq.0) then
    pextra=(xlatt*real(mmax,LONGreal))**2+(ylatt*real(nmax,LONGreal))**2
    vfactr=one
else
    pextra=zero
endif
!
write(nout,'('' minimum x axis lattice number = '',i5/'' maximum x axis lattice number = '',i5)')  &
&                   mmin,mmax
write(nout,'('' minimum y axis lattice number = '',i5/'' maximum y axis lattice number = '',i5)')  &
&                   nmin,nmax
!
!        compute n index for each m index
!
write(nout,'(//'' Bragg reflection points (1/angstroms):''/)')
norder=0
npeak=0
do m=mmin,mmax
    pconst=sqrt(psqr*vfactr**2+pextra-(pin(1)+xlatt*real(m,LONGreal))**2)
    ymin=(-pin(2)-pconst)/ylatt
	  ymax=(-pin(2)+pconst)/ylatt
	  nmin=int(ymin)
	  if(ymin.gt.zero.and.abs(real(nmin,LONGreal)-ymin).gt.small) nmin=nmin+1
	  nmax=int(ymax)
	  if(ymax.lt.zero.and.abs(real(nmax,LONGreal)-ymax).gt.small) nmax=nmax-1
!
!        calculate reflection points cooresponding to this lattice
!        point
!
    qx=pin(1)+xlatt*real(m,LONGreal)
    qxwave=qx*smom/(ten8*hbar)
    do n=nmin,nmax
        qy=pin(2)+ylatt*real(n,LONGreal)
        qzsqr=psqr-qx**2-qy**2
!
!        ******* physically accessible recripocal lattice point ********
!
        if(qzsqr.ge.zero) then
            if(m.eq.0.and.n.eq.0) then
                npnts=nspec
            else
                npnts=nnspec
            endif
            npeak=npeak+1
            mnindx(1,npeak)=m
            mnindx(2,npeak)=n
            qz=sqrt(qzsqr)
            qzwave=qz*smom/(ten8*hbar)
            write(nout,'(/'' Bragg peak at m='',i5,'' n ='',i5,'' wave number='',1pe12.5,          &
&                         '' scaled wave number='',e12.5)') m,n,qzwave,qz
            nnow=2*npnts+2
            if (norder+nnow.gt.order) then
                write(nout,'(/'' Point vector too long in subroutine mkchev, abort''/              &
&                             '' You have real problems, fix them!''/                              &
&                             '' The maximum allowed now is '',i10/)') order
                call abort
            endif
            nstart=norder+1
            nstop=norder+npnts+1
            norder=norder+npnts+2
            twoqz=two*qz
!
!        add in singular point
!
            nsing=norder
            points(nsing)=qz
            wght(nsing)=cmplx(zero,-pt5*pi/qz,LONGreal)
            call rcheby (2,qzsqr,npnts,points(norder+1:norder+npnts),wght(norder+1:norder+npnts))
            norder=norder+npnts
!
!        put the singular point in the appropiate location to
!        have increasing points
!
            if (points(nsing).gt.points(nsing+1)) then
                points(nstart)=points(nsing)
                wght(nstart)=wght(nsing)
                do kk=nsing+2,norder
                    nnow=kk
                    if (points(nsing).lt.points(kk)) go to 40
                enddo
                nnow=nnow+1
40                 do kk=nsing,nnow-2
                    points(kk)=points(kk+1)
                    wght(kk)=wght(kk+1)
                enddo
                points(nnow-1)=points(nstart)
                wght(nnow-1)=wght(nstart)
            endif
!
!       now copy positive partition points into negative ones
!
            kpnt=nstop
            do kk=nstop,nstart,-1
                kpnt=kpnt+1
                points(kk)=-points(kpnt)
                wght(kk)=wght(kpnt)
            enddo
            do kk=nstart,norder
                mindex(kk)=m
                nindex(kk)=n
            enddo
            if(.not.brief) then
                write(nout,'(/'' Physical peak partition for '',''m='',i5,'' n='',i5/)') m,n
                write(nout,'('' index='',i10,'' scaled wave number ='',1pe12.5,                    &
&                            '' quadrature weight ='',2e15.5)') (kk,points(kk),wght(kk),           &
&                            kk=nstart,norder)
            endif
        else
!
!        ************** virtual reciprocal lattice points *************
!
            nnow=nvirtl
            if (norder+2*nnow.gt.order) then
                write(nout,'(/'' Point vector too long in subroutine mkchev, abort''/              &
&                             '' You have real problems, fix them!''/                              &
&                             '' The maximum allowed now is '',i10/)') order
                call abort
            endif
            nstart=norder+1
            nstop=nstart+nnow-1
            norder=norder+nnow
            call rcheby (1,qzsqr,nvirtl,points(norder+1:norder+nvirtl),wght(norder+1:norder+nvirtl))
            norder=norder+nvirtl
            kpnt=nstop
!
!       load negative points
!
            do kk=nstop,nstart,-1
                kpnt=kpnt+1
                points(kk)=-points(kpnt)
                wght(kk)=wght(kpnt)
            enddo
            do kk=nstart,norder
                mindex(kk)=m
                nindex(kk)=n
            enddo
            write(nout,'(/'' Virtual peak partition for m='',i5,'' n='',i5/)') m,n
            if(.not.brief) then
                write(nout,'('' index='',i10,'' scaled wave number ='',1pe12.5,                    &
&                            '' quadrature weight ='',2e15.5)') (kk,points(kk),wght(kk),           &
&                            kk=nstart,norder)
            endif
        endif
    enddo
enddo
!
!        multiply weights by twice the projectile mass and then
!        invert them
!
do kk=1,norder
    wght(kk)=cmplx(one,zero,LONGreal)/(cmplx(two*pmass,zero,LONGreal)*wght(kk))
enddo
!
!        abort if npeak is not equal to nphys
!
if (npeak.ne.nphys) then
    write(nout,'(/'' nphys is not equal to npeak in subroutine mkchev, abort.''/'' npeak='',i5,    &
&                 '' nphys=  '',i5/)') npeak,nphys
    call abort
endif
!
!        abort if norder is not equal to order
!
if (norder.ne.order) then
    write(nout,'(/'' order is not equal to norder in subroutine mkchev, abort.''/'' order='',i5,   &
&                 '' norder=  '',i5/)') order,norder
 
    call abort
endif
!
end


!DIR$ FREE
!CRAY - Add source form directive
subroutine rcheby (type,a,n,point,wght)
!
!
!
!        Copyright (C) 1993 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!        PURPOSE:     Return points and weights for a N-point singular
!                     Chebyshev rule
!        CODED:       F90 version coded 1 May 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Quetzal Computational Associates
!                     3200 Carlisle N.E.
!                     Albuquerque, NM   87110-1664
!
!        INPUT:
!
!        type      integer   =1 if a<0
!                            =2 if a>0
!
!        a         real      the point a mentioned above
!
!        n         integer   number of points to be used in rule
!
!        big       real      constant passed in common /consts/ which
!                            determines the largest number used in the
!                            code.  In this routine, big is used to
!                            determine when a value is to be considered
!                            infinite.
!
!        pi        real      value of pi, passed in common /consts/
!
!        nout      integer   Fortran i/o unit used for printed output
!                            (passed in common /iounit/ ).
!
!
!        OUTPUT:
!
!        point    real a.    real array of length n containing
!                            quadrature points (abscissas)
!
!        wght      complex   complex array of length n containing
!                  array     quadrature weights corresponding to the
!                            points in the point array
!
!        EXTERNAL ROUTINES USED: abort
!
    use kind_definitions
    use constants
    use input_output_units
!
    implicit none
    real (kind=LONGreal), parameter :: zero=0.0,pt5=0.5,one=1.0,two=2.0,three=3.0
    integer :: n,type,i
    real (kind=LONGreal), dimension(n), intent(out) ::  point
    real (kind=LONGreal) a,minusa,value,ratio,ctheta,sine,pt5pi,pt5n,theta
    complex (kind=LONGreal), dimension(n), intent(out) :: wght
    complex (kind=LONGreal) coeff
!
    external abort
!
!        make sure a.gt.0 and finite if type 2.  Also check that n
!        is even
!
    if (type.eq.2) then
        if (a.le.zero.or.a.ge.big) then
            write(nout,'(/'' a is negative or infinite for type 2 in subroutine rcheby, abort.''/  &
&                         '' a='',1pe12.5/)') a
            call abort
        endif
        if (mod(n,2).ne.0) then
            write(nout,'(/'' n is odd in rcheby for type 2, abort'')')
            call abort
        endif
!
!        make sure a.lt.0 if type = 1
!
     elseif (type.eq.1) then
        if (a.ge.zero) then
            write(nout,'(/'' a is positive for type 1 in subroutine rcheby, abort.''/              &
&                        '' a='',1pe12.5/)') a
            call abort
        endif
!
!        unrecognized type, abort
!
    else
        write(nout,'(/'' Illegal type is subroutine rcheby, abort.''/'' The type was '',i5/)') type
        call abort
    endif
!
!***********************************************************************
!        type = 1 (a < 0)
!***********************************************************************
!
    if (type.eq.1) then
        minusa=-a
        value=-pi/(two*real(n,LONGreal)*sqrt(minusa))
        do i=1,n
            ratio=(two*real(i,LONGreal)-one)/(two*real(n,LONGreal))*pi
            point(n-i+1)=sqrt(minusa*(one+cos(ratio))/(one-cos(ratio)))
            wght(n-i+1)=cmplx(value,zero,LONGreal)
        enddo
!
!***********************************************************************
!        type = 2 (a>0)
!***********************************************************************
!
    elseif (type.eq.2) then
        coeff=cmplx(-pi/(real(n,LONGreal)*sqrt(a)),zero,LONGreal)
        ratio=pt5/real(n,LONGreal)
        pt5pi=pt5*pi
        pt5n=pt5*real(n,LONGreal)
        do i=1,n
            theta=ratio*(two*real(i,LONGreal)-one)*pi
            ctheta=cos(theta)
            point(n-i+1)=sqrt(a*(one+ctheta)/(one-ctheta))
            sine=sin((theta+pt5pi)*pt5n)
            wght(n-i+1)=coeff*sine**2/ctheta
        enddo
    endif
!
end subroutine rcheby

subroutine second (tnow)
!
!        Copyright (C) 1993 John K. Prentice
!                           2119 Matthew N.W.
!                           Albuquerque, NM   87104-3210
!                           USA
!
!        PURPOSE:     Return the elapsed system time for this
!                     process on a Sun Microsystems computer.
!        CODED:       F90 version coded 1 May 1993
!        AUTHOR:      John K. Prentice
!        AFFILIATION: Quetzal Computational Associates
!                     3200 Carlisle N.E.
!                     Albuquerque, NM   87110-1664
!
!        INPUT:    none
!
!        OUTPUT:
!
!        tnow      real      elapsed system time (seconds)
!
!
!        EXTERNAL ROUTINES USED: etime
!
!
    USE kind_definitions
!
    implicit none
!
    real (kind=LONGreal) :: tnow
    integer :: count, count_rate
    real (kind=LONGreal) :: total_conformation_time, initial_conformation_time, final_conformation_time,     &
&            percent_conformation_time
    real (kind=LONGreal) :: total_energy_time, initial_energy_time, final_energy_time, percent_energy_time
    real (kind=LONGreal) :: total_time, initial_time, final_time    
!
    call system_clock (count, count_rate)
    if (count_rate == 0) then
        print *,"no system clock, abort."
        stop
    else 
        tnow = real(count,LONGreal)/real(count_rate,LONGreal)
    end if
!
end subroutine second

subroutine abort
print *,"aborting calculation"
stop
end

